#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import sys
from threading import Condition
from PyQt4.QtCore import QObject, QUrl, SIGNAL
from PyQt4.QtGui import QApplication, QPrinter
from PyQt4.QtWebKit import QWebView
 
def renderpdf(url, dest):

    app = QApplication(sys.argv)
    web = QWebView()
    web.load(QUrl(url))

    printer = QPrinter()
    printer.setPageSize(QPrinter.A4)
    printer.setOutputFormat(QPrinter.PdfFormat)
    printer.setOutputFileName(dest)

    def print_file():
        web.print_(printer)
        QApplication.exit()

    QObject.connect(web, SIGNAL("loadFinished(bool)"), print_file)
    app.exec_()

def main():

    if len(sys.argv) != 3:
        sys.stderr.write("Usage: %s URL DEST\n" % sys.argv[0])
        sys.exit(1)

    url = sys.argv[1]
    dest = sys.argv[2]
    renderpdf(url, dest)

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import cherrypy
from cocktail.translations import (
    get_language,
    set_language
)
from cocktail.controllers.parameters import set_cookie_expiration
from woost.models import Site, Language
from woost.controllers.module import Module


class LanguageModule(Module):

    cookie_duration = 60 * 60 * 24 * 15 # 15 days

    def __init__(self, *args, **kwargs):
        Module.__init__(self, *args, **kwargs)
        
    def process_request(self, path):

        language = path.pop(0) if path and path[0] in Language.codes else None
        cherrypy.request.language_specified = (language is not None)

        if language is None:
            language = get_language() or self.infer_language()
        
        cherrypy.response.cookie["language"] = language
        cookie = cherrypy.response.cookie["language"]
        cookie["path"] = "/"
        set_cookie_expiration(cookie, seconds = self.cookie_duration)

        set_language(language)
    
    def infer_language(self):
        # TODO: Parse language headers
        cookie = cherrypy.request.cookie.get("language")
        return cookie.value if cookie else Site.main.default_language

    def translate_uri(self, path = None, language = None):

        qs = ""

        if path is None:
            path = unicode(cherrypy.request.path_info, "utf-8")
            qs = cherrypy.request.query_string

        if language is None:
            language = get_language()

        path_components = path.strip("/").split("/")
        if path_components and path_components[0] in Language.codes:
            path_components.pop(0)

        path_components.insert(0, language)
        return u"/" + u"/".join(path_components) + (u"?" + qs if qs else u"")




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from __future__ import with_statement
import os
import re
import datetime
from time import time, sleep
from subprocess import Popen, PIPE
from tempfile import mkdtemp
from shutil import rmtree
from threading import Lock
from cStringIO import StringIO
from mimetypes import guess_type
from time import mktime, strptime
from httplib import HTTPConnection
from urlparse import urlsplit
from urllib import urlopen
import rfc822
import cherrypy
from cherrypy.lib.cptools import validate_since
from cherrypy.lib.static import serve_file
import Image
import ImageDraw
import ImageFont
import ImageEnhance
import ImageFilter
from cocktail.modeling import (
    cached_getter,
    abstractmethod,
    OrderedDict,
    ListWrapper
)
from cocktail import schema
from cocktail.controllers import (
    context,
    get_parameter,
    Location
)
from woost.models import (
    Item,
    Publishable,
    File,
    URI,
    User,
    get_current_user,
    ReadPermission
)
from woost.controllers.basecmscontroller import BaseCMSController


class ImagesController(BaseCMSController):

    default_format = "PNG"

    formats_by_mime_type = {
        "image/jpeg": "JPEG",
        "image/pjpeg": "JPEG",
        "image/png": "PNG",
        "image/x-png": "PNG",
        "image/gif": "GIF",
        "image/tiff": "TIFF"
    }

    mime_types_by_format = dict(
        (v, k)
        for k, v in formats_by_mime_type.iteritems()
    )

    def __call__(self, id, *processing, **kwargs):
        
        # Get the requested item or content type, validate access
        item = None

        try:
            id = int(id)
        except ValueError:
            for cls in Item.schema_tree():
                if cls.full_name == id:
                    item = cls
                    is_type = True
                    break
        else:
            item = Item.get_instance(int(id))
            is_type = False

        if item is None:
            raise cherrypy.NotFound()

        # Determine the rendering format
        format = kwargs.pop("format", None)
        if format:
            format = format.upper()

        # Find a content renderer that can handle the requested item
        renderer = None
        kind = kwargs.pop("kind", "icon" if is_type else "image").split(",")
        is_icon = False
        
        if "image" in kind:
            if is_type:
                raise ValueError(
                    "Can't set 'kind' to 'image' when rendering a content type"
                )

            for renderer in content_renderers:
                if renderer.can_render(item):
                    break
            else:
                renderer = None

        if renderer is None and "icon" in kind:
            for renderer in icon_renderers:
                if renderer.can_render(item):
                    is_icon = True
                    processing = []
                    break
            else:
                renderer = None

        if renderer is None:
            raise RuntimeError("No content renderer can render %s" % item)

        if not is_type and not is_icon:
            get_current_user().require_permission(ReadPermission, target = item)

        # Caching:
        cache_enabled_param = kwargs.pop("cache", "on")
        if cache_enabled_param == "on":
            cache_enabled = not is_icon
        elif cache_enabled_param == "off":
            cache_enabled = False
        else:
            raise ValueError(
                "Unknown value for the 'cache' parameter: %s"
                % cache_enabled_param
            )

        if cache_enabled:
        
            # Client side cache
            mtime = renderer.last_change_in_appearence(item)
            cherrypy.response.headers["Last-Modified"] = rfc822.formatdate(mtime)
            validate_since()

            # Server side cache            
            extension = format or self.default_format
            file_name = str(id)
            
            if processing:
                file_name += "-" + "-".join(processing)
            
            if kwargs:
                file_name += "?"
                for key in sorted(kwargs):
                    value = kwargs[key]
                    file_name += "%s=%s" % (key, value) 

            file_name = file_name.replace("/", "-") + "." + extension.lower()

            cached_file = os.path.join(
                context["cms"].application_path,
                "thumbnails",
                file_name
            )

            if os.path.exists(cached_file) \
            and os.stat(cached_file).st_mtime >= mtime:
                mime_type = self.mime_types_by_format[extension]
                cherrypy.response.headers["Content-Type"] = mime_type
                return serve_file(cached_file, mime_type)

        # Read rendering parameters
        params = {}
        rendering_schema = renderer.rendering_schema
        
        if rendering_schema:
            
            if not rendering_schema.name:
                raise ValueError("All rendering schemas must have a name")

            get_parameter(
                rendering_schema,
                target = params,
                prefix = rendering_schema.name + ".",
                implicit_booleans = False
            )

        # Render the image
        image = renderer.render(item, **params)

        # The renderer designated an image file as its return value: either
        # serve it unmodified, or load it into memory for further processing
        if isinstance(image, tuple):

            file_name, file_mime_type = image
            file_format = self.formats_by_mime_type[file_mime_type]

            # Image processing or re-encoding requested
            if processing or (format and format != file_format):
                image = Image.open(file_name)
                if not format:
                    format = file_format

            # Unmodified image requested; serve it straight away
            else:
                return serve_file(file_name, file_mime_type)

        # Image processing
        for processor in processing:
            pfunc, pargs, pkwargs = self._parse_processor(processor)
            image = pfunc(image, *pargs, **pkwargs)

        # Encode the resulting image
        buffer = StringIO()

        if not format:
            format = self.default_format

        if format == "JPEG" and image.mode != "RGB":
            image = image.convert("RGB")               

        # Store the resulting image in the server side cache
        if cache_enabled:
            image.save(cached_file, format)

        # Serve the image
        cherrypy.response.headers["Content-Type"] = \
            self.mime_types_by_format[format]
        image.save(buffer, format)
        return buffer.getvalue()

    def _parse_processor(self, processor_string):

        pos = processor_string.find("(")
        if pos == -1:
            proc_name = processor_string
        else:
            proc_name = processor_string[:pos]

        func = image_processors[proc_name]
        args = []
        kwargs = {}
        
        if pos != -1:
            if processor_string[-1] != ")":
                raise ValueError(
                    "Invalid image processor string: %s" % processor_string
                )

            for arg in processor_string[pos + 1:-1].split(","):
                parts = arg.split("=")
                
                # Positional parameters
                if len(parts) == 1:
                    value = self._parse_value(arg)
                    args.append(value)

                # Keyword parameters
                elif len(parts) == 2:
                    key, value = parts
                    value = self._parse_value(value)
                    kwargs[key] = value
                else:
                    raise ValueError(
                        "Invalid image processor string: %s" % processor_string
                    )

        return func, args, kwargs

    def _parse_value(self, value):

        if len(value) >= 2 and value[0] == value[-1] and value[0] in ("'\""):
            return value[1:-1]

        if value in ("True", "False"):
            return bool(value)
        
        if "." in value:
            try:
                value = float(value)
            except ValueError:
                pass
            else:
                return value

        try:
            value = int(value)
        except ValueError:
            pass
        else:
            return value

        raise ValueError(
            "Invalid value for image processor parameter: %s" % value
        )

# Content renderers
#------------------------------------------------------------------------------

class ContentRenderersRegistry(ListWrapper):

    def __init__(self, items = None):
        ListWrapper.__init__(self, items)
        self.__lock = Lock()

    def register(self, renderer, after = None, before = None):
        """Add a new content renderer to the registry.

        By default, renderers are appended at the end of the registry. The
        L{after} or L{before} parameters can be used to specify the position in
        the registry where the renderer should be inserted. This can be
        important, since renderer are tried in registration order

        @param renderer: The renderer to register.
        @type renderer: L{ContentRenderer}

        @param after: 
        """

        with self.__lock:

            marker = after or before

            if marker is None:
                self._items.append(renderer)            
            else:
                if after and before:
                    raise ValueError(
                        "Can't register content renderer '%s' specifying both "
                        "'after' and 'before' parameters"
                        % renderer
                    )

                # Find the requested position
                if isinstance(marker, type):

                    # Find the first instance of the given type
                    for pos, registered_renderer in enumerate(self._items):
                        if isinstance(registered_renderer, marker):
                            break
                    else:
                        pos = -1
                else:
                    # Find an specific instance
                    try:
                        pos = self._items.index(marker)
                    except IndexError:
                        pos = -1

                if pos == -1:
                    raise IndexError(
                        "Trying to register content renderer '%s' %s %s, "
                        "which hasn't been registered yet"
                        % (renderer, "after" if after else "before", marker)
                    )

                if before:
                    self._items.insert(pos, renderer)
                else:
                    self._items.insert(pos + 1, renderer)  


class ContentRenderer(object):

    @abstractmethod
    def can_render(self, item):
        """Indicates if the renderer is able to render the given item.

        @param item: The item to evaluate.
        @type item: L{Item<woost.models.item.Item>}

        @return: True if the renderer claims to be able to render the item,
            False otherwise.
        @rtype: bool
        """

    @cached_getter
    def rendering_schema(self):
        """A schema describing additional parameters for the rendering process.
        @type: L{Schema<cocktail.schema.schema.Schema>}        
        """

    @abstractmethod
    def render(self, item, **kwargs):
        """Produces an image for the given item.

        @param item: The item to render.
        @type item: L{Item<woost.models.item.Item>}

        @param kwargs: Additional parameters that can modify the image produced
            by the renderer. These are implementation dependant, and should
            be described by L{rendering_schema}.

        @return: The image for the given item. If a suitable image file
            exists, the method should return a tuple consisting of a string
            pointing to its path and its MIME type. Otherwise, the method
            should craft the image in-memory and return it as an instance of
            the L{Image<Image.Image>} class.
        @rtype: tuple(str, str) or L{Image<Image.Image>}
        """

    def last_change_in_appearence(self, item):
        """Determines the last time an item was modified in a way that may
        alter its rendering.

        This method is mostly used to check wether cached images are still
        current.

        @param item: The item to evaluate.
        @type item: L{Item<woost.models.item.Item>}

        @return: The timestamp of the last change to the item that could alter
            the resulting image produced by the L{render} method of this
            renderer.
        @rtype: float
        """
        return mktime(item.last_update_time.timetuple())


class ImageFileRenderer(ContentRenderer):
    """A content renderer that handles image files."""

    def can_render(self, item):
        return isinstance(item, File) \
        and item.resource_type == "image" \
        and item.mime_type in ImagesController.formats_by_mime_type

    def render(self, item):
        return (item.file_path, item.mime_type)

    def last_change_in_appearence(self, item):
        return os.stat(item.file_path).st_mtime


class ImageURIRenderer(ContentRenderer):

    def can_render(self, item):
        return isinstance(item, URI) and item.resource_type == "image"

    def get_item_uri(self, item):
        return item.uri

    def render(self, item):
        
        # Open the remote resource
        uri = self.get_item_uri(item)
        http_resource = urlopen(uri)

        # Wrap image data in a buffer
        # (the object returned by urlopen() doesn't support seek(), which is
        # required by PIL)
        buffer = StringIO()
        buffer.write(http_resource.read())
        buffer.seek(0)

        return Image.open(buffer)

    def last_change_in_appearence(self, item):
        uri = self.get_item_uri(item)
        urlparts = urlsplit(uri)
        host = urlparts[1]
        path = urlparts[2] + urlparts[3] + urlparts[4]
        http_conn = HTTPConnection(host)
        http_conn.request("HEAD", path)
        http_date = http_conn.getresponse().getheader("last-modified")
        http_conn.close()
        return mktime(strptime(http_date, "%a, %d %b %Y %H:%M:%S %Z"))


class HTMLRenderer(ContentRenderer):
    """A content renderer that handles XHTML/HTML pages."""

    mime_types = set([
        "text/html",
        "text/xhtml",
        "application/xhtml"
    ])

    def can_render(self, item):
        return (
            isinstance(item, Publishable)
            and item.mime_type in self.mime_types            
            and item.is_accessible(
                user = User.get_instance(qname = "woost.anonymous_user")
            )
        )

    @cached_getter
    def rendering_schema(self):
        return schema.Schema("html", members = [
            schema.Integer("window_width", min = 0),
            schema.Integer("window_height", min = 0)
        ])

    def render(self, item, window_width = None, window_height = None):

        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")
            
            location = Location.get_current_host()
            location.path_info = context["cms"].uri(item)
            
            command = "python -m woost.models.renderurl %s %s" \
                % (unicode(location), temp_image_file)
            
            if window_width is not None:
                command += " --min-width %d" % window_width
            
            if window_height is not None:
                command += " --min-width %d" % window_height 

            Popen(command, shell = True).wait()
            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)


class IconRenderer(ContentRenderer):

    @cached_getter
    def rendering_schema(self):
        return schema.Schema("icon", members = [
            schema.Integer("size", default = 16)
        ])

    def can_render(self, item):
        return True

    def render(self, item, size):
        icon_file = context["cms"].icon_resolver.find_icon(item, size)
        return (icon_file, guess_type(icon_file)[0])


class VideoRenderer(ContentRenderer):
    """A content renderer that handles video files."""

    try:                                                                                                                                                                                                       
        p = Popen(["which", "ffmpeg"], stdout=PIPE)
        ffmpeg_path = p.communicate()[0].replace("\n", "") or None
    except:
        ffmpeg_path = None
    try:
        p = Popen(["which", "grep"], stdout=PIPE)
        grep_path = p.communicate()[0].replace("\n", "") or None
    except:
        grep_path = None
    try:
        p = Popen(["which", "cut"], stdout=PIPE)
        cut_path = p.communicate()[0].replace("\n", "") or None
    except:
        cut_path = None
    try:
        p = Popen(["which", "sed"], stdout=PIPE)
        sed_path = p.communicate()[0].replace("\n", "") or None
    except:
        sed_path = None

    def _secs2time(self, s):
        ms = int((s - int(s)) * 1000000)
        s = int(s)
        # Get rid of this line if s will never exceed 86400
        while s >= 24*60*60: s -= 24*60*60
        h = s / (60*60)
        s -= h*60*60
        m = s / 60
        s -= m*60
        return datetime.time(h, m, s, ms)
    
    def _time2secs(self, d):
        return d.hour*60*60 + d.minute*60 + d.second + \
            (float(d.microsecond) / 1000000)

    @cached_getter
    def rendering_schema(self):
        return schema.Schema("video", members = [
            schema.Integer("position")
        ])

    def can_render(self, item):
        return (
            self.ffmpeg_path 
            and self.grep_path 
            and self.cut_path
            and self.sed_path 
            and isinstance(item, File) 
            and item.resource_type == "video"
        )

    def render(self, item, position = None):
        
        temp_path = mkdtemp()

        try:

            temp_image_file = os.path.join(temp_path, "thumbnail.png")
            
            command1 = "%s -i %s" % (self.ffmpeg_path, item.file_path)
            command2 = "%s Duration | %s -d ' ' -f 4 | %s 's/,//'" % (
                self.grep_path,
                self.cut_path,
                self.sed_path
            )
            p1 = Popen(command1, shell=True, stderr=PIPE)
            p2 = Popen(command2, shell=True, stdin=p1.stderr, stdout=PIPE)
            duration = p2.communicate()[0]

            duration_list = re.split("[.:]", duration)
            video_length = datetime.time(
                int(duration_list[0]), int(duration_list[1]),
                int(duration_list[2]), int(duration_list[3])
            )   

            if position:
                time = self._secs2time(position)
            else:
                seconds = self._time2secs(video_length)
                time = self._secs2time(seconds / 2)
    
            if time > video_length:
                raise ValueError(
                    "Must specify a smaller position than the video duration."
                )
    
            command = u"%s -y -i %s -vframes 1 -ss %s -an -vcodec png -f rawvideo %s " % (
                self.ffmpeg_path, 
                item.file_path, 
                time.strftime("%H:%M:%S"),
                temp_image_file
            )
    
            p = Popen(command.split(), stdout=PIPE)
            p.communicate()
    
            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)


    def last_change_in_appearence(self, item):
        return os.stat(item.file_path).st_mtime


class PDFRenderer(ContentRenderer):
    """A content renderer that handles pdf files."""

    try:
        p = Popen(["which", "convert"], stdout=PIPE)
        convert_path = p.communicate()[0].replace("\n", "") or None
    except:
        convert_path = None

    @cached_getter
    def rendering_schema(self):
        return schema.Schema("pdf", members = [
            schema.Integer("page", default = 0)
        ])

    def can_render(self, item):
        return (
            self.convert_path 
            and isinstance(item, File) 
            and item.resource_type == "document"
            and item.file_name.split(".")[-1].lower() == "pdf"
        )

    def render(self, item, page):
        
        TIMEOUT = 10
        RESOLUTION = 0.25
        temp_path = mkdtemp()

        try:

            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command = u"%s -type TrueColor %s[%d] %s" % ( 
                self.convert_path, item.file_path, page, temp_image_file
            )
            p = Popen(command, shell=True, stdout=PIPE)
            start = time()

            while p.poll() is None:
                if time() - start > TIMEOUT:
                    p.terminate()
                    raise IOError("Timeout was reached")
                sleep(RESOLUTION)

            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)

    def last_change_in_appearence(self, item):
        return os.stat(item.file_path).st_mtime


content_renderers = ContentRenderersRegistry()
content_renderers.register(ImageFileRenderer())
content_renderers.register(ImageURIRenderer())
content_renderers.register(VideoRenderer())
content_renderers.register(PDFRenderer())

# Disabled for performance reasons
# TODO: Provide preferences to enable/disable content renderers?
if False:
    try:
        import PyQt4.QtWebKit
    except ImportError:
        pass
    else:
        content_renderers.register(HTMLRenderer())

icon_renderers = ContentRenderersRegistry()
icon_renderers.register(IconRenderer())

# Image processors
#------------------------------------------------------------------------------
def resolve_px(value, size):
    if isinstance(value, float):        
        value = int(size * value)
    if value < 0:
        value = size + value
    return value

def resolve_color(value):

    if isinstance(value, basestring):

        if len(value) == 3:
            value = tuple(int(d * 2, 16) for d in value)
        elif len(value) == 6:
            value = (
                int(value[0:2], 16),
                int(value[2:4], 16),
                int(value[4:6], 16)
            )
        else:
            raise ValueError("Invalid color: " + value)

    return value

image_processors = OrderedDict()

def image_processor(func):
    image_processors[func.func_name] = func
    return image_processors

@image_processor
def thumbnail(image, width = None, height = None, filter = Image.ANTIALIAS):
    
    im_width, im_height = image.size

    if width is None:
        width = im_width

    if height is None:
        height = im_height

    image.thumbnail(
        (resolve_px(width, im_width),
         resolve_px(height, im_height)),
        filter
    )
    return image

@image_processor
def crop(image, x1, y1, x2, y2):

    width, height = image.size
    return image.crop((
        resolve_px(x1, width),
        resolve_px(y1, height),
        resolve_px(x2, width),
        resolve_px(y2, height)
    ))

@image_processor
def fill(image, width, height, crop = "center", filter = Image.ANTIALIAS):

    source_width, source_height = image.size
    source_ratio = float(source_width) / source_height
    target_ratio = float(width) / height

    if source_ratio > target_ratio:
        target_width = int(height * source_ratio)
        target_height = height
    else:
        target_width = width
        target_height = int(width * (1 / source_ratio))

    image = image.resize((target_width, target_height), filter)

    if crop:
        if crop == "center":
            offset_x = (target_width - width) / 2
            offset_y = (target_height - height) / 2
            image = image.crop((
                offset_x,
                offset_y,
                width + offset_x,
                height + offset_y
            ))
        else:
            raise ValueError("crop = %s not implemented" % crop)

    return image

@image_processor
def rotate(image, angle, expand = True, filter = Image.BICUBIC):
    return image.rotate(angle, filter, expand)

@image_processor
def color(image, level):
    return ImageEnhance.Color(image).enhance(level)

@image_processor
def brightness(image, level):
    return ImageEnhance.Brightness(image).enhance(level)

@image_processor
def contrast(image, level):
    return ImageEnhance.Contrast(image).enhance(level)

@image_processor
def sharpness(image, level):
    return ImageEnhance.Sharpness(image).enhance(level)

@image_processor
def frame(
    image,
    edge = 1,
    edge_color = (0,0,0),
    padding = 0,
    padding_color = (255,255,255)
):
    edge_color = resolve_color(edge_color)
    padding_color = resolve_color(padding_color)

    # Create the canvas
    width, height = image.size
    offset = edge + padding
    canvas = Image.new("RGBA", (width + offset * 2, height + offset * 2))    

    # Paint the border
    if edge:
        canvas.paste(edge_color, None)

    # Paint the padding color
    canvas.paste(
        padding_color,
        (edge,
         edge,
         width + offset * 2 - edge,
         height + offset * 2 - edge)
    )
    
    # Paste the original image over the frame
    canvas.paste(
        image,
        (offset, offset),
        image if image.mode in ("1", "L", "RGBA") else None
    )

    return canvas

@image_processor
def shadow(
    image,
    offset = 5,
    color = (70,70,70),
    padding = 8,
    iterations = 3):   
    
    # Create the backdrop image -- a box in the background colour with a 
    # shadow on it.
    total_width = image.size[0] + abs(offset) + 2 * padding
    total_height = image.size[1] + abs(offset) + 2 * padding
    back = Image.new("RGBA", (total_width, total_height))

    # Place the shadow, taking into account the offset from the image
    shadow_left = padding + max(offset, 0)
    shadow_top = padding + max(offset, 0)
    color = resolve_color(color)
    back.paste(color, [
        shadow_left,
        shadow_top,
        shadow_left + image.size[0], 
        shadow_top + image.size[1]
    ])

    # Apply the filter to blur the edges of the shadow.  Since a small kernel
    # is used, the filter must be applied repeatedly to get a decent blur.
    for n in range(iterations):
        back = back.filter(ImageFilter.BLUR)

    # Paste the input image onto the shadow backdrop
    image_left = padding - min(offset, 0)
    image_top = padding - min(offset, 0)
    back.paste(
        image,
        (image_left, image_top),
        image if image.mode in ("1", "L", "RGBA") else None
    )

    return back

def reduce_opacity(image, opacity):
    # Returns an image with reduced opacity.
    assert opacity >= 0 and opacity <= 1
    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    else:
        image = image.copy()
    alpha = image.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    image.putalpha(alpha)
    return image

@image_processor
def watermark(image, markid, position = "middle", opacity=1):
    
    mark = File.require_instance(int(markid))

    mark_image = Image.open(mark.file_path)

    # Adds a watermark to an image.
    if opacity < 1:
        mark_image = reduce_opacity(mark_image, opacity)
    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    
    # create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = Image.new('RGBA', image.size, (0,0,0,0))
    if position == 'tile':
        for y in range(0, image.size[1], mark_image.size[1]):
            for x in range(0, image.size[0], mark_image.size[0]):
                layer.paste(mark_image, (x, y), mark_image)
    elif position == 'scale':
        # scale, but preserve the aspect ratio
        ratio = min(
            float(image.size[0]) / mark_image.size[0],
            float(image.size[1]) / mark_image.size[1]
        )
        w = int(mark_image.size[0] * ratio)
        h = int(mark_image.size[1] * ratio)
        mark_image = mark_image.resize((w, h))
        layer.paste(
            mark_image,
            ((image.size[0] - w) / 2, (image.size[1] - h) / 2),
            mark_image
        )
    elif position == 'middle':        
        layer.paste(
            mark_image,
            (
                (image.size[0] - mark_image.size[0]) / 2,
                (image.size[1] - mark_image.size[1]) / 2
            ),
            mark_image
        )
    else:
        raise ValueError(
            "Must specify position parameter [tile,scale,middle]."
        )
    #TODO
    # top-left      top         top-right
    # left          middle      right
    # bottom-left   bottom      bottom-right
    # margin

    # composite the watermark with the layer
    return Image.composite(layer, image, layer)




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
import os
from shutil import move
import cherrypy
from cocktail.modeling import getter, cached_getter
from cocktail.events import event_handler
from cocktail import schema
from cocktail.controllers import context
from cocktail.controllers.fileupload import FileUpload
from woost.controllers.backoffice.publishableeditnode \
    import PublishableEditNode


class FileEditNode(PublishableEditNode):

    @cached_getter
    def form_adapter(self):
        adapter = PublishableEditNode.form_adapter(self)
        adapter.exclude(["mime_type"])
        adapter.export_rules.add_rule(ExportUploadInfo())
        adapter.import_rules.add_rule(ImportUploadInfo())
        return adapter

    @cached_getter
    def form_schema(self):
        form_schema = PublishableEditNode.form_schema(self)
        form_schema.add_member(
            FileUpload("upload",
                required = not self.item.is_inserted,
                hash_algorithm = "md5",
                get_file_destination = lambda upload: self.temp_file_path,
                member_group = "content"
            )
        )
        return form_schema

    def iter_changes(self, source = None):
        
        for member, language in PublishableEditNode.iter_changes(self, source):

            # Ignore differences on the upload field if no file has been
            # uploaded
            if member.name == "upload" \
            and schema.get(self.form_data, "upload", None) is None:
                continue

            yield (member, language)

    @getter
    def temp_file_path(self):
        return os.path.join(
            context["cms"].upload_path,
            "temp",
            "%s-%s-%s" % (
                self.item.id,
                cherrypy.session.id,
                self.stack.to_param()
            )
        )

    @event_handler
    def handle_saving(cls, event):

        # Move the uploaded file to its permanent location
        stack_node = event.source
        src = stack_node.temp_file_path

        if os.path.exists(src):
            
            dest = stack_node.item.file_path

            if os.path.exists(dest):
                os.remove(dest)

            move(src, dest)


class ExportUploadInfo(schema.Rule):

    def adapt_object(self, context):

        file_name = context.get("file_name")
        
        if file_name:
            context.set("upload", {
                "file_name": file_name,
                "mime_type": context.get("mime_type"),
                "file_size": context.get("file_size"),
                "file_hash": context.get("file_hash")
            })


class ImportUploadInfo(schema.Rule):

    def adapt_object(self, context):

        context.consume("upload")
        upload = context.get("upload", None)

        if upload:
            context.set("file_name", schema.get(upload, "file_name"))
            context.set("mime_type", schema.get(upload, "mime_type"))
            context.set("file_size", schema.get(upload, "file_size"))
            context.set("file_hash", schema.get(upload, "file_hash"))




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cherrypy.lib.static import serve_file
from woost.controllers import BaseCMSController


class FileController(BaseCMSController):
    """A controller that serves the files managed by the CMS."""

    def __call__(self, disposition = "inline"):
        file = self.context["publishable"]
        
        if disposition not in ("inline", "attachment"):
            raise ValueError("disposition must be either 'inline' or 'attachment', not '%s'" % disposition)
        
        return serve_file(
            file.file_path,
            name = file.file_name,
            content_type = file.mime_type,
            disposition = disposition)


#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import os.path

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import rfc822
import cherrypy
from cherrypy.lib.cptools import validate_since
from pkg_resources import resource_filename, iter_entry_points
from cocktail.events import Event, event_handler
from cocktail.translations import get_language, set_language
from cocktail.controllers import Dispatcher
from cocktail.controllers import Location
from cocktail.controllers.percentencode import percent_encode
from cocktail.persistence import datastore
from woost.models import (
    Item,
    Publishable,
    URI,
    Site,
    ReadPermission,
    ReadTranslationPermission,
    AuthorizationError,
    get_current_user
)
from woost.models.icons import IconResolver
from woost.controllers.basecmscontroller import BaseCMSController
from woost.controllers.language import LanguageModule
from woost.controllers.authentication import (
    AuthenticationModule,
    AuthenticationFailedError
)
from woost.controllers.imagescontroller import ImagesController


class CMS(BaseCMSController):
    
    application_settings = None

    # Application events
    item_saved = Event(doc = """
        An event triggered after an item is inserted or modified.

        @ivar item: The saved item.
        @type item: L{Item<woost.models.Item>}

        @ivar user: The user who saved the item.
        @type user: L{User<woost.models.user.User>}

        @ivar is_new: True for an insertion, False for a modification.
        @type is_new: bool

        @ivar change: The revision describing the changes to the item.
        @type change: L{Change<woost.models.Change>}
        """)

    item_deleted = Event(doc = """
        An event triggered after an item is deleted.

        @ivar item: The deleted item.
        @type item: L{Item<woost.models.Item>}

        @ivar user: The user who deleted the item.
        @type user: L{User<woost.models.user.User>}

        @ivar change: The revision describing the changes to the item.
        @type change: L{Change<woost.models.Change>}
        """)

    producing_output = Event(doc = """
        An event triggered to allow setting site-wide output for controllers.

        @ivar controller: The controller that is producing the output.
        @type controller: L{BaseCMSController
                            <woost.controllers.basecmscontroller
                            .BaseCMSController>}

        @ivar output: The output for the controller. Event handlers can modify
            it as required.
        @type output: dict
        """)

    # Application modules
    LanguageModule = LanguageModule
    AuthenticationModule = AuthenticationModule

    # Webserver configuration
    virtual_path = "/"
    
    # Paths
    application_path = None
    upload_path = None

    # Enable / disable confirmation dialogs when closing an edit session. This
    # setting exists mainly to disable the dialogs on selenium test runs.
    closing_item_requires_confirmation = True

    # A dummy controller for CherryPy, that triggers the cocktail dispatcher.
    # This is done so dynamic dispatching (using the resolve() method of
    # request handlers) can depend on session setup and other requirements
    # being available beforehand.
    class ApplicationContainer(object):

        _cp_config = {
            "tools.sessions.on": True,
            "tools.sessions.storage_type": "file"
        }

        def __init__(self, cms):
            self.__cms = cms
            self.__dispatcher = Dispatcher()            
            app_path = cms.application_path

            if app_path:

                # Set the default location for file-based sessions
                if self._cp_config and \
                self._cp_config.get("tools.sessions.storage_type") == "file":
                    
                    # If the directory doesn't exist, create it
                    session_path = os.path.join(app_path, "sessions")
                    if not os.path.exists(session_path):
                        os.mkdir(session_path)

                    self._cp_config.setdefault(
                        "tools.sessions.storage_path",
                        session_path
                    )

                # Set the default location for uploaded files
                if not cms.upload_path:
                    
                    # If the directory doesn't exist, create it
                    upload_path = os.path.join(app_path, "upload")
                    if not os.path.exists(upload_path):
                        os.mkdir(upload_path)

                    temp_path = os.path.join(upload_path, "temp")
                    if not os.path.exists(temp_path):
                        os.mkdir(temp_path)

                    cms.upload_path = upload_path

        @cherrypy.expose
        def default(self, *args, **kwargs):
            # All requests are forwarded to the nested dispatcher:
            return self.__dispatcher.respond(args, self.__cms)

        # Static resources
        resources = cherrypy.tools.staticdir.handler(
            section = "resources",
            dir = resource_filename("woost.views", "resources")
        )
        
        cocktail = cherrypy.tools.staticdir.handler(
            section = "cocktail",
            dir = resource_filename("cocktail.html", "resources")
        )

    def __init__(self, *args, **kwargs):
    
        BaseCMSController.__init__(self, *args, **kwargs)

        app_path = kwargs.get("application_path")
        if app_path:
            self.application_path = app_path

        self.language = self.LanguageModule(self)
        self.authentication = self.AuthenticationModule(self)
        self.icon_resolver = IconResolver()

        if self.application_path:

            # Add an application specific icon repository
            app_icon_path = os.path.join(
                self.application_path,
                "views", "resources", "images", "icons"
            )
            self.icon_resolver.icon_repositories.insert(0, app_icon_path)

    def run(self, block = True):
                
        cherrypy.tree.mount(
            self.ApplicationContainer(self),
            self.virtual_path,
            self.application_settings
        )
    
        if hasattr(cherrypy.engine, "signal_handler"):
            cherrypy.engine.signal_handler.subscribe()

        if hasattr(cherrypy.engine, "console_control_handler"):
            cherrypy.engine.console_control_handler.subscribe()
    
        cherrypy.engine.start()
        
        if block:
            cherrypy.engine.block()
        else:
            cherrypy.engine.wait(cherrypy.engine.states.STARTED)            

    def resolve(self, path):
        
        # Invoke the language module to set the active language
        self.language.process_request(path)
    
        site = Site.main
        request = cherrypy.request
        unicode_path = [step.decode("utf8") for step in path]

        # Item resolution
        path_resolution = site.resolve_path(unicode_path)

        if path_resolution:
            publishable = path_resolution.item

            for step in path_resolution.matching_path:
                path.pop(0)

            # Redirection to the item's canonical path
            canonical_path = site.get_path(publishable)
            if canonical_path is not None:
                canonical_path = canonical_path.strip("/")
                canonical_path = (
                    canonical_path.split("/")
                    if canonical_path
                    else []
                )
                if canonical_path != path_resolution.matching_path:
                    canonical_uri = "".join(
                        percent_encode(c) for c in "/" + "/".join(
                            step for step in (
                                canonical_path
                                + path_resolution.extra_path
                            )
                        )
                    )
                    if publishable.per_language_publication:
                        canonical_uri = \
                            self.language.translate_uri(canonical_uri)
                    if cherrypy.request.query_string:
                        canonical_uri = canonical_uri + \
                            "?" + cherrypy.request.query_string
                    raise cherrypy.HTTPRedirect(canonical_uri)
        else:
            publishable = site.home
        
        self.context["publishable"] = publishable

        # Controller resolution
        controller = publishable.resolve_controller()

        if controller is None:
            raise cherrypy.NotFound()

        # Add the selected language to the current URI
        if publishable.per_language_publication:
            if not request.language_specified:
                location = Location.get_current()
                location.path_info = "/" + get_language() + location.path_info
                location.go()

        # Remove the language selection from the current URI
        elif request.language_specified:
            location = Location.get_current()
            location.path_info = \
                "/" + "/".join(location.path_info.strip("/").split("/")[1:])
            location.go()

        return controller

    def uri(self, publishable, *args, **kwargs):
        """Obtains the canonical absolute URI for the given item.

        @param publishable: The item to obtain the canonical URI for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param args: Additional path components to append to the produced URI.
        @type args: unicode

        @param kwargs: Key/value pairs to append to the produced URI as query
            string parameters.
        @type kwargs: (unicode, unicode)

        @return: The item's canonical URI, or None if no matching URI can be
            constructed.
        @rtype: unicode
        """
        # User defined URIs
        if isinstance(publishable, URI):
            uri = publishable.uri

        # Regular elements
        else:
            uri = Site.main.get_path(publishable)
            
            if uri is not None:
                
                if publishable.per_language_publication:
                    uri = self.language.translate_uri(uri)

                uri = self.application_uri(uri, *args, **kwargs)
 
        if uri:
            uri = "".join(percent_encode(c) for c in uri)

        return uri

    def translate_uri(self, path = None, language = None):
        return self.application_uri(
            self.language.translate_uri(path = path, language = language)
        )

    def image_uri(self, element, *args, **kwargs):

        if isinstance(element, type):
            element = element.full_name
        elif hasattr(element, "id"):
            element = element.id
        
        return self.application_uri("images", element, *args, **kwargs)

    def icon_uri(self, element, icon_size, thumbnail_size = None):
        
        if thumbnail_size:
            w, h = thumbnail_size
            if w and h:
                args = ["thumbnail(%s,%s)" % thumbnail_size] 
            elif w:
                args = ["thumbnail(width=%s)" % w]
            else:
                args = ["thumbnail(height=%s)" % h]
        else:
            args = []

        kwargs = {"icon.size": str(icon_size)}

        if thumbnail_size:
            kwargs["kind"] = "image,icon"
        elif not isinstance(element, type):
            kwargs["kind"] = "icon"

        return self.image_uri(element, *args, **kwargs)

    def validate_publishable(self, publishable):

        if not publishable.is_published():
            raise cherrypy.NotFound()

        user = get_current_user()
        
        user.require_permission(ReadPermission, target = publishable)
        user.require_permission(
            ReadTranslationPermission,
            language = get_language()
        )

    @event_handler
    def handle_traversed(cls, event):

        datastore.sync()

        cms = event.source
        
        cms.context.update(
            cms = cms,
            publishable = None
        )

        # Set the default language as soon as possible
        language = cms.language.infer_language()
        set_language(language)

        # Invoke the authentication module
        cms.authentication.process_request()

    @event_handler
    def handle_before_request(cls, event):
        
        cms = event.source

        # Validate access to the requested item
        publishable = cms.context["publishable"]
        if publishable is not None:
            cms.validate_publishable(publishable)

            # Set the content type and encoding
            content_type = publishable.mime_type
            if content_type:
                encoding = publishable.encoding
                if encoding:
                    content_type += ";charset=" + encoding
                cherrypy.response.headers["Content-Type"] = content_type

            # TODO: encode / decode the request based on the 'encoding' member?

    @event_handler
    def handle_producing_output(cls, event):
        # Set application wide output parameters
        cms = event.source
        event.output.update(
            cms = cms,
            site = Site.main,
            user = get_current_user(),
            publishable = event.controller.context.get("publishable")
        )

    @event_handler
    def handle_exception_raised(cls, event):

        error = event.exception
        controller = event.source

        content_type = cherrypy.response.headers.get("Content-Type")
        pos = content_type.find(";")

        if pos != -1:
            content_type = content_type[:pos]

        if content_type in ("text/html", "text/xhtml"):

            error_page, status = event.source.get_error_page(error)
            
            if status:
                cherrypy.request.status = status

            if error_page:
                event.handled = True
                
                controller.context.update(
                    original_publishable = controller.context["publishable"],
                    publishable = error_page
                )
                
                response = cherrypy.response
                response.status = status 
                
                error_controller = error_page.resolve_controller()

                # Instantiate class based controllers
                if isinstance(error_controller, type):
                    error_controller = error_controller()
                    error_controller._rendering_format = "html"

                response.body = error_controller()
    
    def get_error_page(self, error):
        """Produces a custom error page for the indicated exception.

        @param error: The exception to describe.
        @type error: Exception

        @return: A tuple comprised of a publishable item to delegate to and an
            HTTP status to set on the response. Either component can be None,
            so that no custom error page is shown, or that the status is not
            changed, respectively.
        @rtype: (L{Document<woost.models.Document>}, int)
        """
        site = Site.main
        is_http_error = isinstance(error, cherrypy.HTTPError)

        # Page not found
        if is_http_error and error.status == 404:
            return site.not_found_error_page, 404
        
        # Access forbidden:
        # The default behavior is to show a login page for anonymous users, and
        # a 403 error message for authenticated users.
        elif is_http_error and error.status == 403 \
        or isinstance(error, (AuthorizationError, AuthenticationFailedError)):
            if get_current_user().anonymous:
                return site.login_page, 200
            else:
                return site.forbidden_error_page, 403
        
        # Generic error
        else:
            return site.generic_error_page, 500

        return None, None

    @event_handler
    def handle_after_request(cls, event):
        datastore.abort()

    def get_file_upload_path(self, id):
        return os.path.join(self.upload_path, str(id))

    images = ImagesController




#-*- coding: utf-8 -*-
"""Provides functions for dealing with user notifications.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import cherrypy

def notify_user(message, category = None, transient = True):
    """Creates a new notification for the current user.
    
    Notifications are stored until a proper page is served to the user. It
    is up to the views to decide how these messages should be displayed.

    @param message: The message that will be shown to the user.
    @type message: unicode

    @param category: An optional free form string identifier that qualifies
        the message. Standard values include 'success' and 'error'.
    @type category: unicode

    @param transient: Indicates if the message should be hidden after a
        short lapse (True), or if it should remain visible until explicitly
        closed by the user (False).
    @type transient: bool
    """
    notifications = cherrypy.session.get("notifications")

    if notifications is None:
        cherrypy.session["notifications"] = notifications = []

    notifications.append((message, category, transient))

def pop_user_notifications():
    """Retrieves pending notification messages that were stored through the
    L{notify_user} method.

    Retrieved messages are considered to be consumed, and therefore they
    are removed from the list of pending notifications.

    @return: The sequence of pending notification messages. Each message
        consists of a tuple with the message text, its category and wether
        or not it should be treated as a transient message.
    @rtype: sequence of (tuple of (unicode, unicode or None, bool))
    """
    notifications = cherrypy.session.get("notifications")
    cherrypy.session["notifications"] = []
    return notifications




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from pkg_resources import iter_entry_points
from cocktail.modeling import classgetter
from cocktail.events import Event
from cocktail.pkgutils import resolve
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from woost.models.item import Item

def load_extensions():
    """Load all available extensions.
    
    This is tipically called during application start up, and follows this
    sequence:

        * New available extensions are installed
        * Previously installed exceptions that are no longer available are
          uninstalled
        * All installed and enabled extensions are initialized
    """
    # Add/remove extensions
    install_new_extensions()
    uninstall_missing_extensions()

    # Load enabled extensions
    NOT_YET_LOADED = object()
    LOADED = object()
    DISABLED = object()
    
    extension_state = {}

    def load(extension):
        state = extension_state.get(extension, NOT_YET_LOADED)

        # Avoid loading an extension twice
        if state is NOT_YET_LOADED:
            
            # Ignore disabled extensions, and those where its implementation is
            # no longer available
            try:
                enabled = extension.enabled
            except AttributeError:
                enabled = False

            if not enabled:
                state = DISABLED
            else:
                # Load dependencies first
                for dependency in extension.dependencies:
                    dependency = resolve(dependency)
                    if isinstance(dependency, type):
                        dependency = list(dependency.select())[0]
                    
                    if not load(dependency):
                        
                        # A dependency for the extension has been disabled; the
                        # extension (and its dependencies) won't be loaded
                        state = DISABLED
                        break
                
                # If all dependencies were loaded, load the extension
                else:
                    extension.loading()
                    state = LOADED

            extension_state[extension] = state
        
        return state is LOADED

    for extension in Extension.select():
        load(extension)

def install_new_extensions():
    """Finds new available extensions and registers them with the site."""

    # Create an instance of each new extension
    installed_extension_types = \
        set(extension.__class__ for extension in Extension.select())

    extensions_installed = False

    for entry_point in iter_entry_points("woost.extensions"):
        extension_type = entry_point.load()
        if extension_type not in installed_extension_types:
            extension = extension_type()
            extension.insert()
            extension.installed()
            extensions_installed = True
        
    if extensions_installed:
        datastore.commit()

def uninstall_missing_extensions():
    """Removes installed extensions that are no longer available."""
    # TODO: implement this!

# Class stub, required by the metaclass
Extension = None

class Extension(Item):
    """Base model for all application extensions.
    
    @ivar dependencies: A sequence of other extensions that should be loaded as
        dependencies of the extension. Each dependency can be specified using a
        reference to an extension object, a reference to an extension class or
        the qualified name of an extension class.
    @type dependencies: sequence of (L{Extension}, L{Extension} subclass or
        str)
    """

    # Add a custom metaclass to hide all subclasses of Extension by default
    class __metaclass__(Item.__metaclass__):

        def __init__(cls, name, bases, members):

            if Extension is not None and "visible_from_root" not in members:
                cls.visible_from_root = False

            Item.__metaclass__.__init__(cls, name, bases, members)

    instantiable = False
    edit_node_class = "woost.controllers.backoffice.extensioneditnode." \
                      "ExtensionEditNode"

    member_order = (
        "extension_author",
        "license",
        "web_page",
        "description",
        "enabled"
    )

    extension_author = schema.String(
        editable = False,
        listed_by_default = False
    )

    license = schema.String(
        editable = False,
        listed_by_default = False
    )

    web_page = schema.String(
        editable = False
    )

    description = schema.String(
        editable = False,
        translated = True
    )

    enabled = schema.Boolean(
        required = True,
        default = False
    )
 
    dependencies = ()

    installed = Event("""An event triggered when an extension is first
        registered with the site.""")

    uninstalled = Event("""An event triggered when an extension is removed from
        the site.""")

    loading = Event("""An event triggered during application start up.""")

    @classgetter
    def instance(cls):
        return iter(cls.select()).next()

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.name)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import cherrypy
from woost.models import Style
from woost.controllers import BaseCMSController


class StylesController(BaseCMSController):

    def __call__(self, backoffice = False):
        
        for style in Style.select():
            declarations = (backoffice and style.admin_declarations) or style.declarations
            yield ".%s{\n%s\n}\n" % (style.class_name, declarations)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from os.path import abspath, dirname, join
from cocktail.html import templates, Element
from cocktail.translations import translations
from cocktail.controllers import context
from woost.models import Style

TinyMCE = templates.get_class("cocktail.html.TinyMCE")

#Required Version: 3.2.2.3 TinyMCE
class RichTextEditor(TinyMCE):

    default_tinymce_params = {
        "plugins": "fullscreen, paste, inlinepopups, advimage, contextmenu, -advimagescale",
        "entity_encoding": "raw",
        "dialog_type": "modal",
        "theme": "advanced",
        "theme_advanced_buttons1_add": "fullscreen",
        "theme_advanced_buttons2_add": "pastetext,pasteword,selectall",
        "theme_advanced_buttons3": "",
        "theme_advanced_toolbar_location": "top",
        "theme_advanced_resizing": True,
        "theme_advanced_statusbar_location": "bottom",
        "theme_advanced_toolbar_align": "left",
        "theme_advanced_path": False,
        "theme_advanced_resize_horizontal": False,
        "document_base_url": "/",
        "relative_urls": False,
        "content_css": "/user_styles/?backoffice=1",
        "fullscreen_settings": {
            "theme_advanced_toolbar_location": "top"
        }
    }

    def _ready(self):

        ctx_uri = context["cms"].contextual_uri()
        current_edit_stack = \
            context["edit_stacks_manager"].current_edit_stack
        
        styles = [
            "%s=%s" % (translations(style), style.class_name)
            for style in Style.select()
        ]

        self.tinymce_params.update(self.default_tinymce_params)

        if current_edit_stack:
            edit_stack_param = current_edit_stack.to_param()
            self.tinymce_params.update(
                init_instance_callback = "initRichTextEditor",
                theme_advanced_styles = ";".join(styles),
                external_image_list_url = "%s/editor_attachments?edit_stack=%s&resource_type=image&language=%s"
                    % (ctx_uri, edit_stack_param, self.language),
                external_link_list_url = "%s/editor_attachments?edit_stack=%s&resource_type=document&language=%s"
                    % (ctx_uri, edit_stack_param, self.language),
                media_external_list_url = "%s/editor_attachments?edit_stack=%s&resource_type=video&language=%s"
                    % (ctx_uri, edit_stack_param, self.language),
            )
        

        load_plugin = Element("script")
        load_plugin["type"] = "text/javascript"
        load_plugin.append("tinymce.PluginManager.load('advimagescale', '/resources/scripts/advimagescale/editor_plugin.js');")
        self.append(load_plugin)
        
        TinyMCE._ready(self)
        
    def __init__(self, *args, **kwargs):
        TinyMCE.__init__(self, *args, **kwargs)
        self.add_resource("/resources/scripts/RichTextEditor.js")




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
import cherrypy
from cocktail import schema
from cocktail.modeling import cached_getter
from cocktail.pkgutils import import_object
from cocktail.controllers import FormControllerMixin, context
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.campaignmonitor import CampaignMonitorExtension
from woost.extensions.campaignmonitor.campaign_monitor_api import \
    CampaignMonitorApi


class CampaignMonitorController(FormControllerMixin, DocumentController):

    campaign_monitor_errors = None

    @cached_getter
    def form_model(self):
        return import_object(self.context["publishable"].subscription_form)

    @cached_getter
    def form_schema(self):
        form_schema = FormControllerMixin.form_schema(self)
        publishable = self.context["publishable"]

        form_schema.add_member(schema.Reference(
            "list",
            type = "woost.extensions.campaignmonitor.campaignmonitorlist."
                "CampaignMonitorList",
            required = True,
            enumeration = publishable.lists,
            default = publishable.lists[0]
        ))
        form_schema.members_order = "name", "email", "list"

        return form_schema

    @cached_getter
    def custom_fields(self):
        return {}

    def submit(self):
        FormControllerMixin.submit(self)

        extension = CampaignMonitorExtension.instance
        cmlist = self.form_data["list"]
        email = self.form_data["email"].encode("utf-8")
        name = self.form_data["name"].encode("utf-8")
 
        api = CampaignMonitorApi(
            extension.api_key,
            extension.client_id
        )

        try:
            user_subscribed = api.subscribers_get_is_subscribed(
                cmlist.list_id,
                email
            )
        except CampaignMonitorApi.CampaignMonitorApiException:
            user_subscribed = False

        # Obtain custom fields
        new_custom_fields = self.custom_fields

        if user_subscribed:
            response = api.subscribers_get_single_subscriber(
                cmlist.list_id,
                email
            )
            subscriber = response.get("Subscribers.GetSingleSubscriber")
            custom_fields = subscriber[0].get("CustomFields")
        else:
            custom_fields = {}

        custom_fields.update(**new_custom_fields)

        # Encode custom fields
        encoded_custom_fields = {}
        for key, value in custom_fields.items():
            encoded_key = (
                key.encode("utf-8") if isinstance(key, unicode) else key
            )
            encoded_value = (
                value.encode("utf-8") if isinstance(value, unicode) else value
            )
            custom_fields[encoded_key] = encoded_value

        try:
            api.subscriber_add_and_resubscribe(
                cmlist.list_id,
                email,
                name,
                encoded_custom_fields
            )
        except CampaignMonitorApi.CampaignMonitorApiException:
            # TODO: Capture the error code and show the correct message
            self.campaign_monitor_errors = True
        else:

            if user_subscribed and cmlist.confirmation_success_page:
                uri = context["cms"].uri(cmlist.confirmation_success_page)
            elif not user_subscribed and cmlist.pending_page:
                uri = context["cms"].uri(cmlist.pending_page)
            else:
                uri = context["cms"].uri(context["publishable"])

            raise cherrypy.HTTPRedirect(uri.encode("utf-8"))

    @cached_getter
    def output(self):

        output = DocumentController.output(self)
        output.update(
            campaign_monitor_errors = self.campaign_monitor_errors
        )

        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail.persistence import datastore
from woost.models import Extension

translations.define("SitemapExtension",
    ca = u"Refinament de l'indexat web",
    es = u"Refinamiento del indexado web",
    en = u"Web crawler hints"
)

translations.define("SitemapExtension-plural",
    ca = u"Refinament de l'indexat web",
    es = u"Refinamiento del indexado web",
    en = u"Web crawler hints"
)


class SitemapExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Genera un mapa de documents per optimitzar l'indexat del lloc
            web, seguint l'estàndard sitemap.org.""",
            "ca"
        )
        self.set("description",            
            u"""Genera un mapa de documentos para optimizar el indexado del
            sitio, siguiendo el estándard sitemap.org.""",
            "es"
        )
        self.set("description",
            u"""Generates a document map to optimize the indexing of the site
            by web crawlers, following the sitemap.org standard.""",
            "en"
        )

    @event_handler
    def handle_loading(cls, event):    
        from woost.models import (
            Site,
            Publishable,
            Document,
            Controller
        )
        from woost.extensions.sitemap import publishable, strings
        
        # Sitemap controller
        sitemap_controller = Controller.get_instance(
            qname = "woost.extensions.sitemap.sitemap_controller"
        )
        if sitemap_controller is None:
            sitemap_controller = Controller()
            sitemap_controller.set("title", u"Controlador de mapa web", "ca")
            sitemap_controller.set("title", u"Controlador de mapa web", "es")
            sitemap_controller.set("title", u"Sitemap controller", "en")
            sitemap_controller.python_name = \
                "woost.extensions.sitemap.sitemapcontroller.SitemapController"
            sitemap_controller.qname = \
                "woost.extensions.sitemap.sitemap_controller"
            sitemap_controller.insert()

        # Sitemap document
        sitemap_doc = Document.get_instance(
            qname = "woost.extensions.sitemap.sitemap_document"
        )
        if sitemap_doc is None:
            sitemap_doc = Document()
            sitemap_doc.set("title", u"Mapa XML del lloc web", "ca")
            sitemap_doc.set("title", u"Mapa XML del sitio web", "es")
            sitemap_doc.set("title", u"XML sitemap", "en")
            sitemap_doc.parent = Site.main.home
            sitemap_doc.path = "sitemap_xml"
            sitemap_doc.per_language_publication = False
            sitemap_doc.mime_type = "text/xml"
            sitemap_doc.hidden = True
            sitemap_doc.sitemap_indexable = False
            sitemap_doc.qname = \
                "woost.extensions.sitemap.sitemap_document"
            sitemap_doc.controller = sitemap_controller
            sitemap_doc.insert()

        # Force indexing of the 'sitemap_indexable' member
        # (can't rely on defaults when executing queries)
        for item in Publishable.select():
            if not hasattr(item, "_sitemap_indexable"):
                item.sitemap_indexable = \
                    item.sitemap_indexable \
                    and not item.hidden

        datastore.commit()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
import cherrypy
from simplejson import dumps
from cocktail.modeling import cached_getter
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail import schema
from cocktail.schema import Reference, String, Integer, Collection
from woost.models import Item
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController, EditNode


class OrderController(BaseBackOfficeController):

    @cached_getter
    def handling_ajax_request(self):
        return self.rendering_format == "json"

    @cached_getter
    def edited_content_type(self):
        return self.item.__class__

    @cached_getter
    def content_type(self):
        return self.member.items.type
    
    @cached_getter
    def collection(self):
        return schema.get(self.stack_node.form_data, self.member)
    
    @cached_getter
    def member(self):
        key = self.params.read(String("member"))
        return self.edited_content_type[key] if key else None

    @cached_getter
    def item(self):
        return self.stack_node.item

    @cached_getter
    def selection(self):
        return self.params.read(
            Collection("selection",
                items = Reference(type = self.content_type)
            )
        )
    
    @cached_getter
    def position(self):
        return self.params.read(
            Integer("position",
                min = 0,
                max = len(self.collection)
            )
        )

    @cached_getter
    def action(self):
        return self.params.read(String("action"))

    @cached_getter
    def ready(self):
        return self.action == "order" \
            and self.selection \
            and self.position is not None

    def submit(self):
        
        def rearrange(collection, items, position):

            size = len(collection)
            
            if position < 0:
                position = size + position

            if position + len(items) > size:
                position = size - len(items)

            for item in items:
                collection.remove(item)
            
            for item in reversed(list(items)):
                collection.insert(position, item)
            
            return collection                   
                   
        rearrange(self.collection, self.selection, self.position)

    def handle_error(self, error):
        if self.handling_ajax_request:
            self.output["error"] = translations(error)
        else:
            BaseBackOfficeController.handle_error(self, error)
    
    @event_handler
    def handle_after_request(cls, event):
        
        controller = event.source

        if not controller.handling_ajax_request:
            if controller.action == "cancel" \
            or (controller.action == "order" and controller.successful):
                if controller.edit_stack:
                    controller.edit_stack.go(-2)
                else:
                    raise cherrypy.HTTPRedirect(controller.relative_uri())

    view_class = "woost.views.BackOfficeOrderView"

    @cached_getter
    def output(self):
        if self.handling_ajax_request:
            return {}
        else:
            output = BaseBackOfficeController.output(self)
            output.update(
                content_type = self.content_type,
                collection = self.collection,
                selection = self.selection
            )
            return output




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""

import random
import string
from cocktail import schema
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail.persistence import datastore
from woost.models import (
    Controller,
    Document,
    EmailTemplate,
    Extension,
    StandardPage,
    Template,
    User
)

translations.define("SignUpExtension",
    ca = u"Alta d'usuaris",
    es = u"Alta de usuarios",
    en = u"Sign Up"
)

translations.define("SignUpExtension-plural",
    ca = u"Altas d'usuaris",
    es = u"Altas de usuarios",
    en = u"Signs Up"
)

class SignUpExtension(Extension):

    # To indicate if the extension was loaded at least one time
    # will be set to True at the end of the first load
    initialized = False

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet als usuaris registrar-se autònomament en el lloc web""",
            "ca"
        )
        self.set("description",            
            u"""Permite a los usuarios registrarse por si mismos en el sitio""",
            "es"
        )
        self.set("description",
            u"""Allows users to register themselves on the site""",
            "en"
        )
        self.secret_key = self._generate_secret_key()

    def _generate_secret_key(self):
        secret_key = ''
        for i in range(0,8):
            secret_key = secret_key + \
                random.choice(
                    string.letters + string.digits
                )
        return secret_key

    @event_handler
    def handle_loading(cls, event):
        
        from woost.extensions.signup.signuppage import SignUpPage
        from woost.extensions.signup.signupcontroller import SignUpController
        from woost.extensions.signup import strings

        extension = event.source

        # Extend User model
        if not hasattr(User, "confirmed_email"):
            User.add_member(
                schema.Boolean(
                    "confirmed_email",
                    default = False,
                    Required = True
                )
            )

        # Make sure the sign up controller exists
        signup_controller = Controller.get_instance(
            qname = u"woost.extensions.signup.signup_controller"
        )
        
        if signup_controller is None:
            signup_controller = Controller()
            signup_controller.python_name = u"woost.extensions.signup.signupcontroller.SignUpController"
            signup_controller.qname = u"woost.extensions.signup.signup_controller"
            signup_controller.set("title", u"Sign Up controller", "en")
            signup_controller.set("title", u"Controlador de alta de usuarios", "es")
            signup_controller.set("title", u"Controlador d'alta d'usuaris", "ca")
            signup_controller.insert()

        # Make sure the sign up confirmation controller exists
        signup_confirmation_controller = Controller.get_instance(
            qname = u"woost.extensions.signup.signup_confirmation_controller"
        )
        
        if signup_confirmation_controller is None:
            signup_confirmation_controller = Controller()
            signup_confirmation_controller.python_name = u"woost.extensions.signup.signupconfirmationcontroller.SignUpConfirmationController"
            signup_confirmation_controller.qname = u"woost.extensions.signup.signup_confirmation_controller"
            signup_confirmation_controller.set("title", u"Sign Up confirmation controller", "en")
            signup_confirmation_controller.set("title", u"Controlador de confirmación alta de usuarios", "es")
            signup_confirmation_controller.set("title", u"Controlador de confirmació d'alta d'usuaris", "ca")
            signup_confirmation_controller.insert()

        # Make sure the sign up template exists
        signup_view = Template.get_instance(
            qname = u"woost.extensions.signup.signup_template"
        )

        if signup_view is None:
            signup_view = Template()
            signup_view.identifier = u"woost.extensions.signup.SignUpView"
            signup_view.engine = u"cocktail"
            signup_view.qname = u"woost.extensions.signup.signup_template"
            signup_view.set("title", u"Sign up view", "en")
            signup_view.set("title", u"Plantilla de alta de usuarios", "es")
            signup_view.set("title", u"Plantilla d'alta d'usuaris", "ca")
            signup_view.insert()

        # Make sure the sign up confirmation template exists
        signup_confirmation_view = Template.get_instance(
            qname = u"woost.extensions.signup.signup_confirmation_template"
        )

        if signup_confirmation_view is None:
            signup_confirmation_view = Template()
            signup_confirmation_view.identifier = u"woost.extensions.signup.SignUpConfirmationView"
            signup_confirmation_view.engine = u"cocktail"
            signup_confirmation_view.qname = u"woost.extensions.signup.signup_confirmation_template"
            signup_confirmation_view.set("title", u"Sign up confirmation template", "en")
            signup_confirmation_view.set("title", u"Plantilla de confirmación de alta de usuarios", "es")
            signup_confirmation_view.set("title", u"Plantilla de confirmació d'alta d'usuaris", "ca")
            signup_confirmation_view.insert()

        # Make sure the default confirmation email template exists
        confirmation_email_template = EmailTemplate.get_instance(
            qname = u"woost.extensions.signup.signup_confirmation_email_template"
        )
        
        if confirmation_email_template is None:
            confirmation_email_template = EmailTemplate()
            confirmation_email_template.python_name = u"woost.extensions.signup.signupconfirmationemailtemplate.SignUpConfirmationEmailTemplate"
            confirmation_email_template.qname = u"woost.extensions.signup.signup_confirmation_email_template"
            # title
            confirmation_email_template.set("title", u"Sign Up Confirmation email template ", "en")
            confirmation_email_template.set("title", u"Plantilla de email de confirmación de alta de usuario", "es")
            confirmation_email_template.set("title", u"Plantilla de correu electrònic de confirmació d'alta d'usuari", "ca")
            # subject
            confirmation_email_template.set("subject", u"User account confirmation", "en")
            confirmation_email_template.set("subject", u"Confirmación de cuenta de usuario", "es")
            confirmation_email_template.set("subject", u"Confirmació de compte d'usuari", "ca")
            # template engine
            confirmation_email_template.template_engine = u"mako"
            # body
            confirmation_email_template.set("body", u"""
                Hello ${user.email}. Click here to confirm your email account.
                <br/> <a href='${confirmation_url}'>${confirmation_url}</a>
            """, "en")
            confirmation_email_template.set("body", u"""
                Hola ${user.email}. Has clic para confirmar tu cuenta de usuario.
                <br/> <a href='${confirmation_url}'>${confirmation_url}</a>
            """, "es")
            confirmation_email_template.set("body", u"""
                Hola ${user.email}. Fes clic per confirmar el teu compte d'usuari.
                <br/> <a href='${confirmation_url}'>${confirmation_url}</a>
            """, "ca")
            # sender
            confirmation_email_template.set("sender", u"'no-reply@woost.info'")
            # receivers
            confirmation_email_template.set("receivers", u"[user.email]")

            confirmation_email_template.insert()

        # Make sure the default confirmation target exists
        confirmation_target = StandardPage.get_instance(
            qname = u"woost.extensions.signup.signup_confirmation_target"
        )
        
        if confirmation_target is None and not extension.initialized:
            confirmation_target = StandardPage()
            confirmation_target.qname = u"woost.extensions.signup.signup_confirmation_target"
            # title
            confirmation_target.set("title", u"Sign Up Confirmation Page", "en")
            confirmation_target.set("title", u"Confirmación de alta de usuario", "es")
            confirmation_target.set("title", u"Confirmació d'alta d'usuari", "ca")
            # Controller
            confirmation_target.controller = signup_confirmation_controller
            confirmation_target.template = signup_confirmation_view
            # parent
#            confirmation_target.parent = Document.get_instance(qname="woost.home")
            # hidden
            confirmation_target.hidden = True

            confirmation_target.insert()

        # Make sure the default signup page exists
        signup_page = SignUpPage.get_instance(
            qname = u"woost.extensions.signup.signup_page"
        )
        
        if signup_page is None and not extension.initialized:
            signup_page = SignUpPage()
            signup_page.qname = u"woost.extensions.signup.signup_page"
            # title
            signup_page.set("title", u"Sign Up", "en")
            signup_page.set("title", u"Registro de usuario", "es")
            signup_page.set("title", u"Registre d'usuari", "ca")
            # User type
            signup_page.user_type = User
            # confirmation target
            signup_page.confirmation_target = confirmation_target
            # parent
            signup_page.parent = Document.get_instance(qname="woost.home")
            # children
            signup_page.children.append(confirmation_target)

            signup_page.insert()

        extension.initialized = True

        datastore.commit()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import re
from cocktail.modeling import abstractmethod
from cocktail import schema
from cocktail.translations import translations
from woost.models.item import Item
from woost.models.publishable import Publishable


class PublicationScheme(Item):
    """Base class for all kinds of content publication schemes."""

    visible_from_root = False
    instantiable = False
    permanent_links = False

    site = schema.Reference(
        type = "woost.models.Site",
        bidirectional = True,
        visible = False
    )

    @abstractmethod
    def resolve_path(self, path):
        """Determines the publishable item that matches the indicated path.       
        
        @param path: The path to evaluate; A list-like object describing a
            a path relative to the application's root.
        @type path: str list

        @return: A structure containing the matching item and its publication
            details. If no matching item can be found, None is returned
            instead.
        @rtype: L{PathResolution}
        """

    @abstractmethod
    def get_path(self, publishable, language):
        """Determines the canonical path of the indicated item.

        @param publishable: The item to get the canonical path for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param language: The language to get the path in. Some schemes produce
            different canonical paths for the same content in different
            languages.
        @type language: str

        @return: The publication path for the indicated item, relative to the
            application's root. If none of the site's publishing schemes can
            produce a suitable path for the item, None is returned instead.
        @rtype: unicode
        """


class PathResolution(object):
    """A structure that provides publication information for an item.

    The L{PublicationScheme.resolve_path} and L{Site.resolve_path} methods use
    this class to wrap their return values.

    @var scheme: The publishing scheme used to resolve the publication details
        for the indicated path. This will only be set when calling
        L{Site.resolve_path}, to enable the caller to discern which of all the
        registered schemes handled the path's resolution.
    @type scheme: L{PublicationScheme}

    @var item: The publishable item that matches the indicated path.
    @type item: L{Publishable<woost.models.publishable.Publishable>}

    @var matching_path: The part of the evaluated path that was used by the
        publishing scheme to determine the matching item.
    @type matching_path: list

    @var extra_path: Additional path components that were not used by the
        publishing scheme when determining the path's matching item. This
        will be relayed to the item's handler when invoking the dispatching
        machinery.
    @type extra_path: list
    """
    scheme = None
    item = None
    matching_path = []
    extra_path = []

    def __init__(self,
        scheme,
        item,
        matching_path = None,
        extra_path = None):

        self.scheme = scheme
        self.item = item
        self.matching_path = matching_path if matching_path is not None else []
        self.extra_path = extra_path if extra_path is not None else []


class HierarchicalPublicationScheme(PublicationScheme):
    """A publication scheme that publishes pages following a name hierarchy.

    The name hierarchy is defined by the L{path<woost.models.Page.path>},
    and L{parent<woost.models.publishable.Publishable.parent>} members of the
    L{Publishable<woost.models.publishable.Publishable>} class.
    """
    instantiable = True

    def resolve_path(self, path):
        
        remaining_path = list(path)
        extra_path = []
        
        while remaining_path:
            page = Publishable.get_instance(
                full_path = u"/".join(remaining_path)
            )
            if page:
                return PathResolution(
                    self,
                    page,
                    remaining_path,
                    extra_path
                )
            extra_path.append(remaining_path.pop())

    def get_path(self, publishable, language):
        return publishable.full_path


class IdPublicationScheme(PublicationScheme):
    """A publication scheme that publishes items based on their id."""
    
    instantiable = True
    permanent_links = True

    def resolve_path(self, path):
        if path:
            try:
                id = int(path[0])
            except:
                pass
            else:
                publishable = Publishable.get_instance(id)
                if publishable is not None:
                    return PathResolution(
                        self,
                        publishable,
                        [path[0]],
                        path[1:]
                    )

    def get_path(self, publishable, language):
        return str(publishable.id)


class DescriptiveIdPublicationScheme(PublicationScheme):
    """A publication scheme that combines a unique identifier and a descriptive
    text fragment.

    @ivar word_separator: The character used to separate words on an item's
        title when crafting its descriptive fragment.
    @type word_separator: str

    @ivar id_regexp: The regular expression used to extract the unique
        identifier from an URI. The expression must define an 'id' named group. 
    @type id_regexp: regular expression
    
    @param format: A python formatting string, used for composing an item's
        URI from its unique identifier and flattened title. Receives 'title'
        and 'id' parameters.
    @type format: str
    """
    instantiable = True
    permanent_links = True
    
    members_order = [
        "id_separator",
        "word_separator",
        "id_regexp",
        "title_splitter_regexp",
        "format"
    ]

    id_separator = schema.String(
        required = True,
        default = "_"
    )

    word_separator = schema.String(
        required = True,
        default = "-"
    )

    id_regexp = schema.Member(
        required = True,
        default = re.compile(r"(.+_)?(?P<id>\d+)$"),
        normalization = lambda value:
            re.compile(value) if isinstance(value, basestring) else value,
        serialize_request_value = lambda value:
            value.pattern if value else value
    )

    title_splitter_regexp = schema.Member(
        required = True,
        default = re.compile(r"\W+", re.UNICODE),
        normalization = lambda value:
            re.compile(value, re.UNICODE)
            if isinstance(value, basestring)
            else value,
        serialize_request_value = lambda value:
            value.pattern if value else value
    )

    format = schema.String(
        required = True,
        default = "%(title)s%(separator)s%(id)d"
    )

    _uri_encodings = ["utf-8", "iso-8859-1"]

    def resolve_path(self, path):
        
        if path:

            ref = path[0]
            
            # Try to decode the supplied URI using a selection of different
            # string encodings
            if not isinstance(ref, unicode):
                for encoding in self._uri_encodings:
                    try:
                        ref = ref.decode(encoding)
                    except UnicodeDecodeError, ex:
                        pass
                    else:
                        break
                else:
                    raise ex

            # Discard descriptive text
            match = self.id_regexp.match(ref)

            if match is None:
                return None
            else:
                id = match.group("id")

            try:
                id = int(id)
            except:
                return None

            publishable = Publishable.get_instance(id)
            
            if publishable is not None:
                return PathResolution(
                    self,
                    publishable,
                    [path[0]],
                    path[1:]
                )
        
    def get_path(self, publishable, language):
        
        title = translations(publishable, language)

        if title:
            title = self.title_splitter_regexp.sub(
                self.word_separator,
                title
            )
            title = title.lower()            
            ref = self.format % {
                "title": title,
                "id": publishable.id,
                "separator": self.id_separator
            }
        else:
            ref = str(publishable.id)

        return ref




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail.html import Element
from cocktail.translations import translations
from cocktail.controllers import context


class ItemLabel(Element):

    item = None
    icon_size = 16
    icon_visible = True
    thumbnail = True

    def _ready(self):
        Element._ready(self)

        if self.item:

            for schema in self.item.__class__.descend_inheritance(True):
                self.add_class(schema.name)

            if self.icon_visible:
                self.append(self.create_icon())

            self.append(self.get_label())

    def create_icon(self):
        img = Element("img")
        img.add_class("icon")
        img["src"] = context["cms"].icon_uri(
            self.item if self.item.is_inserted else self.item.__class__, 
            self.icon_size,
            (24, None) if self.thumbnail and self.item.is_inserted else None
        )
        return img
    
    def get_label(self):
        return translations(self.item)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from datetime import datetime
from cocktail.modeling import getter
from cocktail.events import event_handler
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import (
    PersistentObject, PersistentClass, datastore, PersistentMapping
)
from woost.models.changesets import ChangeSet, Change
from woost.models.action import Action

# Extension property that allows changing the controller that handles a
# collection in the backoffice
schema.Collection.edit_controller = \
    "woost.controllers.backoffice.collectioncontroller." \
    "CollectionController"

# Extension property that makes it easier to customize the edit view for a
# collection in the backoffice
schema.Collection.edit_view = None


class Item(PersistentObject):
    """Base class for all CMS items. Provides basic functionality such as
    authorship, group membership, draft copies and versioning.
    """
    members_order = [
        "id",
        "qname",
        "author",
        "owner"
    ]

    # Extension property that indicates if content types should be visible from
    # the backoffice root view
    visible_from_root = True

    def __translate__(self, language, **kwargs):
        if self.draft_source is not None:
            return translations(
                "woost.models.Item draft copy",
                language,
                item = self.draft_source,
                draft_id = self._draft_id,
                **kwargs
            )
        else:
            return PersistentObject.__translate__(self, language, **kwargs)

    # Unique qualified name
    #--------------------------------------------------------------------------
    qname = schema.String(
        unique = True,
        indexed = True,
        searchable = False,
        listed_by_default = False,
        member_group = "administration"
    )

    # Backoffice customization
    #--------------------------------------------------------------------------
    show_detail_view = "woost.views.BackOfficeShowDetailView"
    show_detail_controller = \
        "woost.controllers.backoffice.showdetailcontroller." \
        "ShowDetailController"
    collection_view = "woost.views.BackOfficeCollectionView"
    edit_node_class = "woost.controllers.backoffice.editstack.EditNode"
    edit_view = "woost.views.BackOfficeFieldsView"
    edit_form = "woost.views.ContentForm"
    edit_controller = \
        "woost.controllers.backoffice.itemfieldscontroller." \
        "ItemFieldsController"

    # Indexing
    #--------------------------------------------------------------------------
 
    # Make sure draft copies' members don't get indexed
    def _should_index_member(self, member):
        return PersistentObject._should_index_member(self, member) and (
            member.primary or self.draft_source is None
        )

     # When validating unique members, ignore conflicts with the draft source
    def _counts_as_duplicate(self, other):
        return PersistentObject._counts_as_duplicate(self, other) \
            and other is not self.draft_source

    # Versioning
    #--------------------------------------------------------------------------
    changes = schema.Collection(
        required = True,
        versioned = False,
        editable = False,
        items = "woost.models.Change",
        bidirectional = True,
        visible = False
    )

    creation_time = schema.DateTime(
        versioned = False,
        editable = False,
        member_group = "administration"
    )

    last_update_time = schema.DateTime(
        versioned = False,
        editable = False,
        member_group = "administration"
    )

    is_draft = schema.Boolean(
        required = True,
        default = False,
        indexed = True,
        listed_by_default = False,
        editable = False,
        versioned = False,
        member_group = "administration"
    )

    draft_source = schema.Reference(
        type = "woost.models.Item",
        related_key = "drafts",
        bidirectional = True,
        editable = False,
        listed_by_default = False,
        indexed = True,
        versioned = False,
        member_group = "administration"
    )

    drafts = schema.Collection(
        items = "woost.models.Item",
        related_key = "draft_source",
        bidirectional = True,
        cascade_delete = True,
        editable = False,
        versioned = False,
        member_group = "administration"
    )

    _draft_count = 0
    _draft_id = None

    @getter
    def draft_id(self):
        """A numerical identifier for draft copies, guaranteed to be unique
        among their source item.
        @type: int
        """
        return self._draft_id

    def make_draft(self):
        """Creates a new draft copy of the item. Subclasses can tweak the copy
        process by overriding either this method or L{get_draft_adapter} (for
        example, to exclude one or more members).

        @return: The draft copy of the item.
        @rtype: L{Item}
        """
        draft = self.__class__()
        draft.draft_source = self
        draft.is_draft = True
        draft.bidirectional = False
        
        self._draft_count += 1
        draft._draft_id = self._draft_count

        adapter = self.get_draft_adapter()
        adapter.export_object(
            self,
            draft,
            source_schema = self.__class__,
            source_accessor = schema.SchemaObjectAccessor,
            target_accessor = schema.SchemaObjectAccessor
        )
        
        return draft

    def confirm_draft(self):
        """Confirms a draft. On draft copies, this applies all the changes made
        by the draft to its source element, and deletes the draft. On brand new
        drafts, the item itself simply drops its draft status, and otherwise
        remains the same.
        
        @raise ValueError: Raised if the item is not a draft.
        """            
        if not self.is_draft:
            raise ValueError("confirm_draft() must be called on a draft")

        if self.draft_source is None:
            self.bidirectional = True
            self.is_draft = False
        else:
            adapter = self.get_draft_adapter()
            adapter.source_accessor = schema.SchemaObjectAccessor
            adapter.target_accessor = schema.SchemaObjectAccessor
            adapter.import_object(
                self,
                self.draft_source,
                source_schema = self.__class__
            )
            self.delete()

    @classmethod
    def get_draft_adapter(cls):
        """Produces an adapter that defines the copy process used by the
        L{make_draft} method in order to produce draft copies of the item.

        @return: An adapter with all the rules required to obtain a draft copy
            of the item.
        @rtype: L{Adapter<cocktail.schema.adapter.Adapter>}
        """
        adapter = schema.Adapter()
        adapter.collection_copy_mode = schema.shallow
        adapter.exclude([
            member.name
            for member in cls.members().itervalues()
            if cls._should_exclude_in_draft(member)
        ] + ["owner"])
        return adapter

    @classmethod
    def _should_exclude_in_draft(cls, member):
        return not member.editable or not member.visible

    @classmethod
    def _create_translation_schema(cls, members):
        members["versioned"] = False
        PersistentObject._create_translation_schema.im_func(cls, members)
        
    @classmethod
    def _add_member(cls, member):
        if member.name == "translations":
            member.editable = False
            member.searchable = False
        PersistentClass._add_member(cls, member)

    def _get_revision_state(self):
        """Produces a dictionary with the values for the item's versioned
        members. The value of translated members is represented using a
        (language, translated value) mapping.

        @return: The item's current state.
        @rtype: dict
        """

        # Store the item state for the revision
        state = PersistentMapping()

        for key, member in self.__class__.members().iteritems():
           
            if not member.versioned:
                continue

            if member.translated:
                value = dict(
                    (language, translation.get(key))
                    for language, translation in self.translations.iteritems()
                )
            else:
                value = self.get(key)

            state[key] = value

        return state

    # Item insertion overriden to make it versioning aware
    @event_handler
    def handle_inserting(cls, event):

        item = event.source
        now = datetime.now()
        item.creation_time = now
        item.last_update_time = now

        if not item.is_draft:
            changeset = ChangeSet.current

            if changeset:
                change = Change()
                change.action = Action.get_instance(identifier = "create")
                change.target = item
                change.changed_members = set(
                    member.name
                    for member in item.__class__.members().itervalues()
                    if member.versioned
                )
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change
                
                if item.author is None:
                    item.author = changeset.author

                if item.owner is None:
                    item.owner = changeset.author
                
                change.insert(event.inserted_objects)
        
    # Extend item modification to make it versioning aware
    @event_handler
    def handle_changed(cls, event):

        item = event.source

        if getattr(item, "_v_initializing", False) \
        or not event.member.versioned \
        or not item.is_inserted \
        or item.is_draft:
            return
        
        changeset = ChangeSet.current

        if changeset:

            member_name = event.member.name
            language = event.language
            change = changeset.changes.get(item.id)

            if change is None:
                action_type = "modify"
                change = Change()
                change.action = Action.get_instance(identifier = action_type)
                change.target = item
                change.changed_members = set()
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change
                item.last_update_time = datetime.now()
                change.insert()
            else:
                action_type = change.action.identifier

            if action_type == "modify":
                change.changed_members.add(member_name)
                
            if action_type in ("create", "modify"):
                if language:
                    change.item_state[member_name][language] = event.value
                else:
                    change.item_state[member_name] = event.value
        else:
            item.last_update_time = datetime.now()

    @event_handler
    def handle_deleting(cls, event):
        
        item = event.source

        # Update the last time of modification for the item
        item.last_update_time = datetime.now()

        # Break the relation to the draft's source. This needs to be done
        # explicitly, because drafts are flagged as non bidirectional (to keep
        # their changes in isolation), which prevents automatic management of
        # referential integrity
        if item.draft_source is not None:
            item.draft_source.drafts.remove(item)

        if not item.is_draft:
            changeset = ChangeSet.current

            # Add a revision for the delete operation
            if changeset:
                change = changeset.changes.get(item.id)

                if change and change.action.identifier != "delete":
                    del changeset.changes[item.id]

                if change is None \
                or change.action.identifier not in ("create", "delete"):
                    change = Change()
                    change.action = Action.get_instance(identifier = "delete")
                    change.target = item
                    change.changeset = changeset
                    changeset.changes[item.id] = change
                    change.insert()
    
    _preserved_members = frozenset([changes])

    def _should_cascade_delete(self, member):
        return member.cascade_delete and not self.is_draft

    def _should_erase_member(self, member):
        return PersistentObject._should_erase_member(self, member) \
            and member not in self._preserved_members

    # Ownership and authorship
    #--------------------------------------------------------------------------
    author = schema.Reference(
        indexed = True,
        editable = False,
        type = "woost.models.User",
        listed_by_default = False,
        member_group = "administration"
    )
    
    owner = schema.Reference(
        indexed = True,
        type = "woost.models.User",
        listed_by_default = False,
        member_group = "administration"
    )


Item.id.versioned = False
Item.id.editable = False
Item.id.listed_by_default = False
Item.id.member_group = "administration"
Item.changes.visible = False




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import buffet
import smtplib
from email.mime.text import MIMEText
from email.Header import Header
from email.Utils import formatdate
from cocktail import schema
from cocktail.html.datadisplay import display_factory
from woost.models import Item, Site


class EmailTemplate(Item):

    visible_from_root = False
    encoding = "utf-8"

    members_order = [
        "title",
        "mime_type",
        "sender",
        "receivers",
        "bcc",
        "template_engine",
        "subject",
        "body"
    ]

    title = schema.String(
        listed_by_default = False,
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        translated = True
    )

    mime_type = schema.String(
        required = True,
        default = "html",
        listed_by_default = False,
        enumeration = [
            "plain",
            "html"
        ]
    )

    sender = schema.String(
        edit_control = display_factory(
            "cocktail.html.CodeEditor", syntax = "python"
        )
    )

    receivers = schema.String(
        required = True,
        edit_control = display_factory(
            "cocktail.html.CodeEditor", syntax = "python"
        )
    )
    
    bcc = schema.String(
        listed_by_default = False,
        edit_control = display_factory(
            "cocktail.html.CodeEditor", syntax = "python"
        )
    )

    template_engine = schema.String(
        enumeration = buffet.available_engines.keys()
    )

    subject = schema.String(
        translated = True,
        edit_control = "cocktail.html.TextArea"
    )

    body = schema.String(    
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea"
    )

    def send(self, context = None):

        smtp_host = Site.main.smtp_host or "localhost"
        smtp_port = smtplib.SMTP_PORT
        smtp_user = Site.main.smtp_user
        smtp_password = Site.main.smtp_password

        if context is None:
            context = {}
        
        def eval_member(key):
            expr = self.get(key)
            return eval(expr, context.copy()) if expr else None

        # MIME block
        mime_type = self.mime_type
        pos = mime_type.find("/")

        if pos != -1:
            mime_type = mime_type[pos + 1:]

        # Subject and body (templates)
        if self.template_engine:
            template_engine = buffet.available_engines[self.template_engine]
            engine = template_engine(
                options = {"mako.output_encoding": self.encoding}
            )

            def render(field_name):
                markup = self.get(field_name)
                if markup:
                    template = engine.load_template(
                        "EmailTemplate." + field_name,
                        self.get(field_name)
                    )
                    return engine.render(context, template = template)                    
                else:
                    return u""
           
            subject = render("subject").strip()
            body = render("body")
        else:
            subject = self.subject.encode(self.encoding)
            body = self.body.encode(self.encoding)
            
        message = MIMEText(body, _subtype = mime_type, _charset = self.encoding)

         # Receivers (python expression)
        receivers = eval_member("receivers")
        if receivers:
            receivers = set(r.strip() for r in receivers) 

        if not receivers:
            return set()
 
        message["To"] = ", ".join(receivers)

        # Sender (python expression)
        sender = eval_member("sender")
        if sender:
            message["From"] = sender

        # BCC (python expression)
        bcc = eval_member("bcc")
        if bcc:
            receivers.update(r.strip() for r in bcc)

        if subject:
            message["Subject"] = Header(subject, self.encoding)

        message["Date"] = formatdate()

        # Send the message
        smtp = smtplib.SMTP(smtp_host, smtp_port)
        if smtp_user and smtp_password:
            smtp.login(
                smtp_user.encode(self.encoding), 
                smtp_password.encode(self.encoding)
            )
        smtp.sendmail(sender, list(receivers), message.as_string())
        smtp.quit()

        return receivers

    def __translate__(self, language, **kwargs):
        return (self.draft_source is None and self.get("title", language)) \
            or Item.__translate__(self, language, **kwargs)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
import os
from subprocess import Popen, PIPE
from cocktail import schema
from cocktail.controllers import context
from cocktail.modeling import abstractmethod
from cocktail.translations import translations
from woost.models import Item


class StaticSiteSnapShooter(Item):
    """A class tasked with creating a static snapshot of a site's content to
    a concrete location.
    
    This is mostly an abstract class, meant to be extended by subclasses. Each
    subclass should implement the static snapshot creation. 
    """
    instantiable = False
    visible_from_root = False
    integral = True

    destination = schema.Reference(
        type = "woost.extensions.staticsite.staticsitedestination.StaticSiteDestination",
        bidirectional = True,
        required = True,
        related_key = "snapshooter"
    )

    url = schema.String(
        required = True
    )

    @abstractmethod
    def snapshoot(self):
        """ Walks the static snapshoot of a site's content 

        @return: The file's contents, and their snapshoot's relative path. The contents can be                                                                                                                                      
        specified using a file-like object, or a filesystem path.
        @rtype: (file-like or str, str) generator
        """
    
    def __translate__(self, language, **kwargs):
        return (self.draft_source is None and self.url) \
            or Item.__translate__(self, language, **kwargs)


class WgetSnapShooter(StaticSiteSnapShooter):
    """ A class that creates a static snapshoot of a site's content using wget """
    instantiable = True

    file_names_mode = schema.String(                                          
        required = True,
        default = "unix",
        enumeration = frozenset(("unix", "windows")),
        translate_value = lambda value, **kwargs:    
            u"" if not value else translations(
                "woost.extensions.staticsite.staticsitesnapshooter.WgetSnapShooter.file_names_mode " + value,
                **kwargs
            )
    )

    def snapshoot(self):

        snapshoot_path = os.path.join(
            context["cms"].application_path,
            u"snapshoots",
            str(self.id)
        )

        cmd = "wget --mirror --page-requisites --html-extension \
            --convert-links --no-parent --no-host-directories \
            --directory-prefix=%s --restrict-file-names=%s %s"

        cmd = cmd % (snapshoot_path, self.file_names_mode, self.url)

        p = Popen(cmd, shell=True, stdout=PIPE)
        p.communicate()

        for root, dirs, files in os.walk(snapshoot_path):
            for file in files:
                file_path = os.path.join(root, file)
                relative_path = os.path.relpath(file_path, snapshoot_path)
                yield (file_path, relative_path)
    



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from datetime import datetime
from cocktail.modeling import getter
from cocktail.events import event_handler
from cocktail.pkgutils import import_object
from cocktail.translations import translations, get_language
from cocktail import schema
from cocktail.schema.expressions import Expression
from woost.models.item import Item
from woost.models.usersession import get_current_user
from woost.models.permission import ReadPermission
from woost.models.permission import ReadPermission, PermissionExpression


class Publishable(Item):
    """Base class for all site elements suitable for publication."""
    
    instantiable = False

    # Backoffice customization
    preview_view = "woost.views.BackOfficePreviewView"
    preview_controller = "woost.controllers.backoffice." \
        "previewcontroller.PreviewController"
    edit_node_class = "woost.controllers.backoffice.publishableeditnode." \
        "PublishableEditNode"
 
    groups_order = ["navigation", "presentation", "publication"]

    members_order = [
        "mime_type",
        "resource_type",
        "encoding",
        "controller",
        "parent",
        "path",
        "full_path",
        "hidden",
        "enabled",
        "translation_enabled",
        "start_date",
        "end_date"
    ]

    mime_type = schema.String(
        required = True,
        default = "text/html",
        text_search = False,
        format = r"^[^/]+/[^/]+$",
        listed_by_default = False,
        member_group = "presentation"
    )

    resource_type = schema.String(
        indexed = True,
        editable = False,
        enumeration = (
            "document",
            "image",
            "audio",
            "video",
            "package",
            "html_resource",
            "other"
        ),
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.models.Publishable.resource_type " + value,
                 language,
                **kwargs
            ),
        listed_by_default = False,
        member_group = "presentation"
    )

    encoding = schema.String(
        listed_by_default = False,
        member_group = "presentation",
        default = "utf-8"
    )

    controller = schema.Reference(
        type = "woost.models.Controller",
        bidirectional = True,
        listed_by_default = False,
        member_group = "presentation"
    )

    def resolve_controller(self):
        if self.controller and self.controller.python_name:
            return import_object(self.controller.python_name)

    parent = schema.Reference(
        type = "woost.models.Document",
        bidirectional = True,
        related_key = "children",
        listed_by_default = False,
        member_group = "navigation"
    )

    path = schema.String(
        max = 1024,
        indexed = True,
        listed_by_default = False,
        member_group = "navigation"
    )

    full_path = schema.String(
        indexed = True,
        unique = True,
        editable = False,
        member_group = "navigation"
    )
    
    hidden = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "navigation"
    )

    per_language_publication = schema.Boolean(
        required = True,
        default = False,
        indexed = True,
        visible = False,
        member_group = "publication"
    )

    enabled = schema.Boolean(
        required = True,
        default = True,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    translation_enabled = schema.Boolean(
        required = True,
        default = True,
        translated = True,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    start_date = schema.DateTime(
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date,
        listed_by_default = False,
        member_group = "publication"
    )

    @event_handler
    def handle_changed(cls, event):

        member = event.member
        publishable = event.source

        if member.name == "path":
            publishable._update_path(publishable.parent, event.value)

        elif member.name == "parent":
            publishable._update_path(event.value, publishable.path)

        elif member.name == "mime_type":
            publishable.resource_type = get_category_from_mime_type(event.value)

    def _update_path(self, parent, path):

        parent_path = parent and parent.full_path

        if parent_path and path:
            self.full_path = parent_path + "/" + path
        else:
            self.full_path = path

    def ascend_tree(self, include_self = False):
        """Iterate over the item's ancestors, moving towards the root of the
        document tree.

        @param include_self: Indicates if the object itself should be included
            in the iteration.
        @type include_self: bool

        @return: An iterable sequence of pages.
        @rtype: L{Document} iterable sequence
        """
        publishable = self if include_self else self.parent
        while publishable is not None:
            yield publishable
            publishable = publishable.parent

    def descends_from(self, page):
        """Indicates if the object descends from the given document.

        @param page: The hypothetical ancestor of the page.
        @type page: L{Document<woost.models.document.Document>}

        @return: True if the object is contained inside the given document or
            one of its descendants, or if it *is* the given document. False in
            any other case.
        @rtype: bool
        """
        ancestor = self

        while ancestor is not None:
            if ancestor is page:
                return True
            ancestor = ancestor.parent

        return False

    @getter
    def resources(self):
        """Iterates over all the resources that apply to the item.        
        @type: L{Publishable}
        """
        ancestry = list(self.ascend_tree(include_self = False))
        ancestry.reverse()

        for document in ancestry:
            for resource in document.branch_resources:
                yield resource

    def is_current(self):
        now = datetime.now()
        return (self.start_date is None or self.start_date <= now) \
            and (self.end_date is None or self.end_date > now)
    
    def is_published(self, language = None):
        return (
            not self.is_draft
            and (
                self.get("translation_enabled", language)                
                if self.per_language_publication
                else self.enabled
            )
            and self.is_current()
        )

    def is_accessible(self, user = None, language = None):
        return self.is_published(language) \
            and (user or get_current_user()).has_permission(
                ReadPermission,
                target = self
            )

    @classmethod
    def select_accessible(cls, *args, **kwargs):
        return cls.select(filters = [
            IsAccessibleExpression(get_current_user())
        ]).select(*args, **kwargs)


class IsPublishedExpression(Expression):
    """An expression that tests if items are published."""

    def eval(self, context, accessor = None):
        return context.is_published()

    def resolve_filter(self, query):

        def impl(dataset):

            # Exclude disabled items
            simple_pub = (
                Publishable.per_language_publication.index[False]
                .intersection(Publishable.enabled.index[True])
            )
            per_language_pub = (
                Publishable.per_language_publication.index[True]
                .intersection(Publishable.translation_enabled.index[
                    (get_language(), True)
                ])
            )
            dataset.intersection_update(simple_pub | per_language_pub)

            # Exclude drafts
            is_draft_expr = Item.is_draft.equal(False)
            dataset = is_draft_expr.resolve_filter(query)[1](dataset)
            
            # Publication min/max date
            now = datetime.now()
                
            s = Publishable.start_date.index
            e = Publishable.end_date.index

            # No start date set, or the start date has been reached
            dataset.intersection_update(
                s[None] | set(s.values(max = now))
            )
            
            # No end date set, or the end date hasn't been reached yet
            dataset.intersection_update(
                e[None] | set(e.values(min = now, excludemin = True))
            )

            return dataset
        
        return ((-1, 1), impl)


class IsAccessibleExpression(Expression):
    """An expression that tests that items can be accessed by a user.
    
    The expression checks both the publication state of the item and the
    read permissions for the specified user.

    @ivar user: The user that accesses the items.
    @type user: L{User<woost.models.user.User>}
    """
    def __init__(self, user = None):
        Expression.__init__(self)
        self.user = user

    def eval(self, context, accessor = None):
        return context.is_accessible(user = self.user)

    def resolve_filter(self, query):

        def impl(dataset):
            access_expr = PermissionExpression(
                self.user or get_current_user(),
                ReadPermission
            )
            published_expr = IsPublishedExpression()
            dataset = access_expr.resolve_filter(query)[1](dataset)
            dataset = published_expr.resolve_filter(query)[1](dataset)
            return dataset
        
        return ((-1, 1), impl)


mime_type_categories = {}

for category, mime_types in (
    ("text/plain", ("text",)),
    ("html_resource", (
        "text/css",
        "text/javascript",
        "text/ecmascript",
        "application/javascript",
        "application/ecmascript"
    )),
    ("document", (
        "application/vnd.oasis.opendocument.text",
        "application/vnd.oasis.opendocument.spreadsheet",
        "application/vnd.oasis.opendocument.presentation",
        "application/msword",
        "application/msexcel",
        "application/msaccess",
        "application/mspowerpoint",
        "application/mswrite",
        "application/vnd.ms-excel",
        "application/vnd.ms-access",
        "application/vnd.ms-powerpoint",
        "application/vnd.ms-project",
        "application/vnd.ms-works",
        "application/vnd.ms-xpsdocument",
        "application/rtf",
        "application/pdf",
        "application/x-pdf",
        "application/postscript",
        "application/x-latex",
        "application/vnd.oasis.opendocument.database"
    )),
    ("package", (
        "application/zip",
        "application/x-rar-compressed",
        "application/x-tar",
        "application/x-gtar",
        "application/x-gzip",
        "application/x-bzip",
        "application/x-stuffit",
        "vnd.ms-cab-compressed"
    ))
):
    for mime_type in mime_types:
        mime_type_categories[mime_type] = category

def get_category_from_mime_type(mime_type):
    """Obtains the file category that best matches the indicated MIME type.
    
    @param mime_type: The MIME type to get the category for.
    @type mime_type: str

    @return: A string identifier with the category matching the indicated
        MIME type (one of 'document', 'image', 'audio', 'video', 'package',
        'html_resource' or 'other').
    @rtype: str
    """
    pos = mime_type.find("/")

    if pos != -1:
        prefix = mime_type[:pos]

        if prefix in ("image", "audio", "video"):
            return prefix
    
    return mime_type_categories.get(mime_type, "other")




#-*- coding: utf-8 -*-
u"""
Provides the CMS subclass used to customize the behavior of the site.
"""
import cherrypy
from pkg_resources import resource_filename
from woost.controllers.application import CMS


class _PROJECT_NAME_CMS(CMS):

    application_path = resource_filename("_PROJECT_MODULE_", None)

    _cp_config = CMS.copy_config()
    _cp_config["rendering.engine"] = "_TEMPLATE_ENGINE_"
    _cp_config["rendering.engine_options"] = {
        "mako.directories": [
            resource_filename("_PROJECT_MODULE_", "views"),
            resource_filename("woost", "views")
        ],
        "mako.output_encoding": "utf-8"
    }

    class ApplicationContainer(CMS.ApplicationContainer):        
        _PROJECT_MODULE__resources = cherrypy.tools.staticdir.handler(
            section = "_PROJECT_MODULE__resources",
            dir = resource_filename("_PROJECT_MODULE_.views", "resources")
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""

# Application server configuration
import cherrypy
cherrypy.config.update({
    "global": {
        "server.socket_host": "_WEBSERVER_HOST_",
        "server.socket_port": _WEBSERVER_PORT_,
        "tools.encode.on": True,
        "tools.encode.encoding": "utf-8",
        "tools.decode.on": True,
        "tools.decode.encoding": 'utf-8'
    }
})

# Object store provider
from cocktail.persistence import datastore
from ZEO.ClientStorage import ClientStorage
db_host = "_DATABASE_HOST_"
db_port = _DATABASE_PORT_
datastore.storage = lambda: ClientStorage((db_host, db_port))



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.controllers.renderingengines import get_rendering_engine
from woost.controllers import BaseCMSController


class DocumentController(BaseCMSController):
    """A controller that serves rendered pages."""

    @cached_getter
    def page_template(self):
        template = self.context["publishable"].template

        if template is None:
            raise cherrypy.NotFound()

        return template

    @cached_getter
    def rendering_engine(self):
        engine_name = self.page_template.engine

        if engine_name:
            return get_rendering_engine(engine_name)
        else:
            return BaseCMSController.rendering_engine(self)

    @cached_getter
    def view_class(self):
        return self.page_template.identifier




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from __future__ import with_statement
from woost.tests.models.basetestcase import BaseTestCase


class DraftTestCase(BaseTestCase):

    def test_indexing_enabled_for_new_drafts(self):

        from cocktail import schema
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)
 
        foo = Foo()
        foo.is_draft = True
        foo.spam = 4
        foo.insert()
        assert set(Foo.spam.index.items()) == set([(4, foo.id)])

    def test_indexing_disabled_for_draft_copies(self):
        
        from cocktail import schema
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)
 
        source = Foo()
        source.spam = 4
        source.insert()
        assert set(Foo.spam.index.items()) == set([(4, source.id)])

        foo = Foo()
        foo.draft_source = source
        foo.spam = 3
        foo.insert()

        assert set(Foo.spam.index.items()) == set([(4, source.id)])
        
        foo.spam = 2
        assert set(Foo.spam.index.items()) == set([(4, source.id)])

    def test_versioning_disabled_for_drafts(self):

        from woost.models import Document, changeset_context

        with changeset_context() as insertion_cs:
            doc = Document()
            doc.is_draft = True
            doc.insert()
        
        assert not insertion_cs.changes

        with changeset_context() as modification_cs:
            doc.set("title", "Hello, world!", "en")
        
        assert not modification_cs.changes

        with changeset_context() as deletion_cs:
            doc.delete()
        
        assert not deletion_cs.changes

    def test_copy_draft(self):

        from woost.models import Document

        # Create a source item
        doc = Document()
        doc.set("title", "Test item", "en") 
        doc.insert()

        # Create a draft for the item
        draft = doc.make_draft()
        draft.insert()
        assert draft.is_draft
        assert draft.draft_source is doc
        assert draft in doc.drafts
        assert not doc.id == draft.id
        assert doc.get("title", "en") == draft.get("title", "en")
 
        # Modify the draft
        draft.set("title", "Modified test item", "en")
        assert not doc.get("title", "en") == draft.get("title", "en")
        draft.confirm_draft()
        
        # Confirm it
        assert draft.id not in Document.index
        assert not draft in doc.drafts
        assert doc.get("title", "en") == "Modified test item"




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from __future__ import with_statement
from ZODB.POSException import ConflictError
from cocktail.modeling import getter, cached_getter, InstrumentedSet
from cocktail.schema import (
    String,
    Collection,
    Reference,
    RelationMember,
    remove
)
from cocktail.persistence import datastore, PersistentClass
from woost.models import (
    changeset_context,
    delete_validating,
    get_current_user,
    DeletePermission
)
from woost.controllers.backoffice.editstack import EditNode
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class DeleteController(BaseBackOfficeController):
    
    MAX_TRANSACTION_ATTEMPTS = 3

    _delete_dry_run_done = False

    def __init__(self, *args, **kwargs):
        BaseBackOfficeController.__init__(self, *args, **kwargs)

        self._delete_tree = []
        self._blocking_members = []

    @cached_getter
    def selection(self):
        """The selected subset of items that should be deleted.
        @type: L{Item<woost.models.item.Item>} collection
        """
        return self.params.read(
            Collection("selection", items = "woost.models.Item"))

    def _delete_dry_run(self):

        if self._delete_dry_run_done:
            return

        visited = set()

        def recurse(item, container):

            if item in visited:
                return
            
            visited.add(item)
            deleted_descendants = list()
            container.append((item, deleted_descendants))
            
            for member in item.__class__.members().itervalues():

                if isinstance(member, RelationMember) \
                and member.related_type \
                and isinstance(member.related_type, PersistentClass):
                    
                    if member.block_delete and item.get(member):
                        self._blocking_members.append((item, member))

                    if item._should_cascade_delete(member):
                        value = item.get(member)

                        if value is not None:
                            if isinstance(member, Reference):
                                value = (value,)

                            member_container = list()
                            for descendant in value:
                                if descendant is not None:
                                    recurse(descendant, member_container)
       
                            if len(member_container) > 0:
                                deleted_descendants.append(
                                    (member, member_container)
                                )

        for item in self.selection:
            recurse(item, self._delete_tree)
        
        self._delete_dry_run_done = True

    @getter
    def delete_tree(self):
        self._delete_dry_run()
        return self._delete_tree

    @getter
    def blocking_members(self):
        self._delete_dry_run()
        return self._blocking_members

    @cached_getter
    def action(self):
        """A string identifier indicating the action that has been activated by
        the user.
        @type: str or None
        """
        return self.params.read(String("action"))

    @cached_getter
    def submitted(self):
        return self.action is not None
    
    def submit(self):
        if self.action == "confirm_delete":
            
            # Load the edit stack before deleting any item, to ensure its
            # loaded properly
            stack = self.edit_stack

            user = get_current_user()

            for i in range(self.MAX_TRANSACTION_ATTEMPTS):

                deleted_set = None

                with changeset_context(author = user):
                    for item in self.selection:
                        deleted_set = delete_validating(
                            item,
                            deleted_set = deleted_set
                        )

                try:
                    datastore.commit()
                except ConflictError:
                    datastore.abort()
                    datastore.sync()
                else:
                    break       
          
            # Purge the edit stack of references to deleted items
            if stack:
                stack.remove_references(deleted_set)
            
            # Launch CMS.item_deleted events
            cms = self.context["cms"]

            for item in deleted_set:
                cms.item_deleted(
                    item = item,
                    user = user,
                    change = item.changes[-1] if item.changes else None
                )               
            
            self.go_back()
            
        elif self.action == "cancel":
            self.go_back()

    view_class = "woost.views.BackOfficeDeleteView"

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output["delete_tree"] = self.delete_tree
        output["blocking_members"] = self.blocking_members
        return output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.translations import translations, get_language
from cocktail.html.element import Element
from cocktail.html import templates
from cocktail.controllers import context
from woost.models import Language

LinkSelector = templates.get_class("cocktail.html.LinkSelector")


class LanguageSelector(LinkSelector):

    translated_labels = True
    tag = "ul"

    def create_entry(self, value, label, selected):        
        
        entry = Element("li")
        link = self.create_entry_link(value, label)

        if selected:
            strong = Element("strong")
            strong.append(link)
            entry.append(strong)
        else:
            entry.append(link)

        return entry

    def _ready(self):
        
        if self.items is None:
            self.items = Language.codes

        if self.value is None:
            self.value = get_language()

        LinkSelector._ready(self)

        # Hack for IE <= 6
        if self.children:
            self.children[-1].add_class("last")

    def get_item_label(self, language):
        if self.translated_labels:
            return translations(language, language)
        else:
            return translations(language)

    def get_entry_url(self, language):
        cms = context["cms"]
        return cms.translate_uri(language = language)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2009
"""
from cocktail.modeling import cached_getter
from cocktail import schema
from woost.controllers.backoffice.editstack import EditNode


class UserEditNode(EditNode):

    @cached_getter
    def form_adapter(self):

        form_adapter = EditNode.form_adapter(self)
        form_adapter.exclude("password_confirmation")
        
        if self.item.is_inserted:
            form_adapter.exclude("change_password")
            form_adapter.copy("password",
                export_condition = False,
                import_condition = lambda context:
                    context.get("change_password", default = None)
            )

        return form_adapter

    @cached_getter
    def form_schema(self):
        
        form_schema = EditNode.form_schema(self)
        password_member = form_schema.get_member("password")
        
        if password_member:

            order = form_schema.members_order = list(form_schema.members_order)
            pos = order.index("password")

            password_conf_member = schema.String(
                name = "password_confirmation",            
                edit_control = "cocktail.html.PasswordBox",
                visible_in_detail_view = False,
                required = password_member.required
            )
            form_schema.add_member(password_conf_member)
            order.insert(pos + 1, "password_confirmation")

            if self.item.is_inserted:

                change_password_member = schema.Boolean(
                    name = "change_password",
                    required = True,
                    default = False,
                    visible_in_detail_view = False
                )
                form_schema.add_member(change_password_member)
                order.insert(pos, "change_password")
                
                password_member.exclusive = change_password_member
                password_conf_member.exclusive = change_password_member
        
            @form_schema.add_validation
            def validate_password_confirmation(form_schema, value, ctx):
                password = ctx.get_value("password")               
                password_confirmation = ctx.get_value("password_confirmation")

                if password and password_confirmation \
                and password != password_confirmation:
                    yield PasswordConfirmationError(
                            form_schema, value, ctx)

        return form_schema

    def iter_changes(self):
        
        # Discard differences in the password field
        for member, language in EditNode.iter_changes(self):
            if member.name not in (
                "change_password", "password", "password_confirmation"
            ):
                yield (member, language)


class PasswordConfirmationError(schema.exceptions.ValidationError):
    """A validation error produced when the password and its confirmation field
    are given different values."""




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
from cocktail.html import templates, Element
from cocktail.controllers import context
from woost.models import Site

TreeView = templates.get_class("cocktail.html.TreeView")


class Menu(TreeView):
    """A visual component used to render navigation menus and site maps.
    
    @var root_visible: If set to True, the produced tree has a single root.
        Otherwise, the root element is hidden and the tree appears to have
        multiple roots instead.
    @type root_visible: bool

    @var selection: Indicates the selected item.
    @type selection: L{Publishable<woost.models.publishable.Publishable>}

    @var emphasized_selection: When set to True, adds a <strong> tag to
        highlight the selected item.
    @type emphasized_selection: bool

    @var linked_selection: Indicates if the entry for the selected item should
        behave as a link or not.
    @type linked_selection: bool
    
    @var linked_containers: Indicates if entries that contain other entries
        should behave as links.
    @type linked_containers: bool

    @var max_depth: The maximum depth for the menu.
    @type max_depth: int

    @var expanded: When set to True, the menu will be completely expanded. When
        False, entries will only be expanded if they contain the selected
        entry.
    @type expanded: bool
    """
    root_visible = False
    selection = None
    emphasized_selection = True
    linked_selection = True
    linked_containers = True
    max_depth = None
    expanded = False

    def _ready(self):

        self._cms = context["cms"]

        if self.root is None:
            self.root = Site.main.home

        if self.selection is None:
            self.selection = context["publishable"]
        
        # Find the selected path
        self._expanded = set()
        item = self.selection

        while item is not None:
            self._expanded.add(item)
            item = item.parent

        # Limited depth
        self._depth = 2 if self.root_visible else 1

        TreeView._ready(self)
    
    def filter_item(self, item):
        return not item.hidden and item.is_accessible()

    def get_item_uri(self, item):
        return self._cms.uri(item)

    def create_entry(self, item):
        entry = TreeView.create_entry(self, item)

        if item in self._expanded:
            entry.add_class("selected")

        return entry

    def create_label(self, item):
        
        if self.should_link(item):
            label = Element("a")
            label["href"] = self.get_item_uri(item)
        else:
            label = Element("span")
        
        label.append(self.get_item_label(item))

        if self.emphasized_selection and item is self.selection:
            if label.tag == "a":
                label = Element("strong", children = [label])
            else:
                label.tag = "strong"

        return label

    def should_link(self, item):
        return (self.linked_selection or item is not self.selection) \
            and (self.linked_containers or not item.children)

    def get_child_items(self, parent):

        if (self.max_depth is not None and self._depth > self.max_depth) \
        or (not self.expanded and parent not in self._expanded):
            return []
        else:
            return TreeView.get_child_items(self, parent)

    def _fill_children_container(self, container, item, children):
        self._depth += 1
        TreeView._fill_children_container(self, container, item, children)
        self._depth -= 1        




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import re
import sys
from time import time
from simplejson import loads
from urllib import urlopen
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import Extension, Language

translations.define("CountriesExtension",
    ca = u"Països",
    es = u"Países",
    en = u"Countries"
)

translations.define("CountriesExtension-plural",
    ca = u"Països",
    es = u"Países",
    en = u"Countries"
)

translations.define("CountriesExtension.update_frequency",
    ca = u"Freqüència d'actualització",
    es = u"Frecuencia de actualización",
    en = u"Update frequency"
)

translations.define("CountriesExtension.update_frequency-explanation",
    ca = u"El nombre de dies entre actualitzacions de la llista de paisos",
    es = u"El número de días entre actualizaciones de la lista de países",
    en = u"Number of days between updates to the country list"
)

SECONDS_IN_A_DAY = 60 * 60 * 24


class CountriesExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona accés a la llista de països del món,
            actualitzada a través d'Internet.""",
            "ca"
        )
        self.set("description",            
            u"""Proporciona acceso a la lista de países del mundo, actualizada
            a través de Internet.""",
            "es"
        )
        self.set("description",
            u"""Provides a list of world countries, automatically updated from
            the Internet.""",
            "en"
        )

    last_update = None

    update_frequency = schema.Integer(
        min = 1,
        required = True,
        default = 15
    )

    @event_handler
    def handle_loading(cls, event):
        
        from woost.extensions.countries import country, strings

        now = time()
        ext = event.source
        
        if ext.last_update is None \
        or now - ext.last_update >= ext.update_frequency * SECONDS_IN_A_DAY:
            try:
                ext.update_country_list()
            except:
                pass
            else:
                ext.last_update = now
                datastore.commit()

    def update_country_list(self):
        from woost.extensions.countries.country import Country
        error = None
        
        database_modified = False
        service_uri = "http://www.lonelydrops.com/drops/1.0/list/%s/countries"
    
        data_expr = re.compile(
            r"var\s+drops_countries_[a-z]{2}\s*=\s*([^;]+)",
            re.DOTALL
        )
        json_property_normalization = re.compile(
            r"([a-zA-Z_$][a-zA-Z_$0-9]*)(?=\s*:)"
        )

        for language in Language.codes:
            try:
                javascript = urlopen(service_uri % language).read()
            except Exception, error:
                pass
            else:
                match = data_expr.search(javascript)

                if not match:
                    sys.stderr.write(
                        "The country list provider returned data in an "
                        "unexpected format (requested translation: %s)" % language
                    )
                    continue

                json = match.group(1)
                json = json_property_normalization.sub(r'"\1"', json)
                
                try:
                    data = loads(json)
                except Exception, error:
                    sys.stderr.write(
                        "The country list provider returned invalid data: %s"
                        "(requested translation: %s)\n" % (error, language)
                    )
                    continue

                for country_data in data:

                    # Ignore continents
                    if country_data.get("isParent"):
                        continue

                    name = country_data["label"]
                    iso_code = country_data["value"].lower()
                    country = Country.get_instance(iso_code = iso_code)

                    # New country
                    if country is None:
                        country = Country()
                        country.iso_code = iso_code
                        country.set("country_name", name, language)
                        country.insert()
                        database_modified = True

                    # Modified country
                    elif country.get("country_name", language) != name:
                        country.set("country_name", name, language)
                        database_modified = True
        
        if error:
            raise error

        return database_modified




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""

from cocktail import schema
from cocktail.events import event_handler
from cocktail.persistence import datastore
from cocktail.translations import translations
from woost.models import (
    Site,
    StandardPage,
    Language,
    Controller,
    Template,
    Extension, 
    get_current_user,
    CreatePermission,                                                                                                                                                                                          
    ModifyPermission,
    DeletePermission,
    PermissionExpression
)

translations.define("CampaignMonitorExtension",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

translations.define("CampaignMonitorExtension-plural",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

class CampaignMonitorExtension(Extension):

    initialized = False

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet la integració amb el sistema de mailing Campaign Monitor""",
            "ca"
        )
        self.set("description",            
            u"""Permite la integración con el sistema de mailing Campaign Monitor""",
            "es"
        )
        self.set("description",
            u"""Allows the integration with the Campaign Monitor mailing system""",
            "en"
        )


    @event_handler
    def handle_loading(cls, event):

        extension = event.source

        from woost.extensions.campaignmonitor import (
            campaignmonitorlist,
            strings,
            useraction
        )

        from woost.extensions.campaignmonitor.campaignmonitorsubscriptionpage \
            import CampaignMonitorSubscriptionPage

        # Setup the synchronization view
        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController
        from woost.extensions.campaignmonitor.synccontroller import \
            SyncCampaignMonitorListsController

        BackOfficeController.sync_campaign_monitor_lists = \
            SyncCampaignMonitorListsController

        # Extension fields
        from woost.extensions.campaignmonitor.campaignmonitorlist \
            import CampaignMonitorList

        CampaignMonitorExtension.add_member(
            schema.String(
                "api_key",
                required = True
            )
        )

        CampaignMonitorExtension.add_member(
            schema.String(
                "client_id",
                required = True
            )
        )

        CampaignMonitorExtension.add_member(
            schema.Collection(
                "lists",
                items = schema.Reference(
                    type = CampaignMonitorList
                ),
                integral = True,
                related_end = schema.Reference()
            )   
        )

        site_languages = Language.codes
            
        # Subscription controller
        campaign_monitor_controller = Controller.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_controller"
        )   

        if campaign_monitor_controller is None:
            campaign_monitor_controller = Controller()
            campaign_monitor_controller.python_name = \
                u"woost.extensions.campaignmonitor.campaignmonitorcontroller.CampaignMonitorController"
            campaign_monitor_controller.qname = u"woost.extensions.campaignmonitor.subscription_controller"
            if "en" in site_languages:
                campaign_monitor_controller.set(
                    "title", u"Subscription to Campaign Monitor controller", "en"
                )
            if "es" in site_languages:
                campaign_monitor_controller.set(
                    "title", u"Controlador de suscripicón a Campaign Monitor", "es"
                )
            if "ca" in site_languages:
                campaign_monitor_controller.set(
                    "title", u"Controlador de subscripció a Campaign Monitor", "ca"
                )
            campaign_monitor_controller.insert()

        # Unsubscription controller
        unsubscription_controller = Controller.get_instance(
            qname = u"woost.extensions.campaignmonitor.unsubscription_controller"
        )   

        if unsubscription_controller is None:
            unsubscription_controller = Controller()
            unsubscription_controller.python_name = \
                u"woost.extensions.campaignmonitor.campaignmonitorunsubscriptioncontroller.CampaignMonitorUnsubscriptionController"
            unsubscription_controller.qname = u"woost.extensions.campaignmonitor.unsubscription_controller"
            if "en" in site_languages:
                unsubscription_controller.set(
                    "title", u"Unsubscription to Campaign Monitor controller", "en"
                )
            if "es" in site_languages:
                unsubscription_controller.set(
                    "title", u"Controlador de baja de Campaign Monitor", "es"
                )
            if "ca" in site_languages:
                unsubscription_controller.set(
                    "title", u"Controlador de baixa de Campaign Monitor", "ca"
                )
            unsubscription_controller.insert()

        # Subscription template
        subscription_view = Template.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_template"
        )

        if subscription_view is None:
            subscription_view = Template()
            subscription_view.identifier = \
                u"woost.extensions.campaignmonitor.CampaignMonitorSubscriptionView"
            subscription_view.engine = u"cocktail"
            subscription_view.qname = u"woost.extensions.campaignmonitor.subscription_template"
            if "en" in site_languages:
                subscription_view.set("title", u"Subscription to Campaign Monitor view", "en")
            if "es" in site_languages:
                subscription_view.set("title", u"Vista de suscripción a Campaign Monitor", "es")
            if "ca" in site_languages:
                subscription_view.set("title", u"Vista de subscripció a Campaign Monitor", "ca")
            subscription_view.insert()
        
        # Unsubscription template
        unsubscription_view = Template.get_instance(
            qname = u"woost.extensions.campaignmonitor.unsubscription_template"
        )

        if unsubscription_view is None:
            unsubscription_view = Template()
            unsubscription_view.identifier = \
                u"woost.extensions.campaignmonitor.CampaignMonitorUnsubscriptionView"
            unsubscription_view.engine = u"cocktail"
            unsubscription_view.qname = u"woost.extensions.campaignmonitor.unsubscription_template"
            if "en" in site_languages:
                unsubscription_view.set("title", u"Unsubscription from Campaign Monitor view", "en")
            if "es" in site_languages:
                unsubscription_view.set("title", u"Vista de baja de Campaign Monitor", "es")
            if "ca" in site_languages:
                unsubscription_view.set("title", u"Vista de baixa de Campaign Monitor", "ca")
            unsubscription_view.insert()
        
        # Default subscription page
        subscription_page = CampaignMonitorSubscriptionPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_page"
        )

        if subscription_page is None and not extension.initialized:
            subscription_page = CampaignMonitorSubscriptionPage(
                qname = u"woost.extensions.campaignmonitor.subscription_page"
            )
            if "en" in site_languages:
                subscription_page.set("title", u"Subscription page", "en")
            if "ca" in site_languages:
                subscription_page.set("title", u"Pàgina de subscripció", "ca")
            if "es" in site_languages:
                subscription_page.set("title", u"Página de suscripción", "es")
            subscription_page.parent = Site.main.home
            subscription_page.hidden = True
            subscription_page.insert()

        # Standard Template
        standard_template = Template.get_instance(
            identifier = "woost.views.StandardView"
        )

        # Default pending page
        pending_page = StandardPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.pending_page"
        )

        if pending_page is None and not extension.initialized:
            pending_page = StandardPage(
                qname = u"woost.extensions.campaignmonitor.pending_page"
            )
            if "en" in site_languages:
                pending_page.set("title", u"Page with instructions for subscription", "en")
                pending_page.set("body", u"""We've just been sent an email to 
confirm your email address. Please click on the link in this email to confirm 
your subscription.""", "en")
            if "es" in site_languages:
                pending_page.set("title", u"Pàgina d'instruccions de subscripció", "ca")
                pending_page.set("body", u"""Te hemos enviado un e-mail para 
confirmar tu dirección de correo electrónico. Por favor, haz clic en el enlace 
que aparece en el e-mail para confirmar tu suscripción.""", "es")
            if "ca" in site_languages:
                pending_page.set("title", u"Página de instrucciones de suscripcion", "es")
                pending_page.set("body", u"""T'hem enviat un e-mail per confirmar 
la teva adreça de correu electrònic. Si et plau, fes clic a l'enllaç que 
apareix a l'e-mail per confirmar la teva subscripció.""", "ca")
            pending_page.parent = Site.main.home
            pending_page.template = standard_template
            pending_page.hidden = True
            pending_page.insert()

        # Default confirmation success page
        confirmation_success_page = StandardPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.confirmation_success_page"
        )

        if confirmation_success_page is None and not extension.initialized:
            confirmation_success_page = StandardPage(
                qname = u"woost.extensions.campaignmonitor.confirmation_success_page"
            )
            if "en" in site_languages:
                confirmation_success_page.set("title", u"Confirmation page", "en")
                confirmation_success_page.set("body", u"""Your subscription has 
been confirmed. You've been added to our list and will hear from us soon.""", "en")
            if "ca" in site_languages:
                confirmation_success_page.set("title", u"Pàgina de confirmació", "ca")
                confirmation_success_page.set("body", u"""La teva subscripció ha 
sigut confirmada. La teva adreça ha sigut afegida a la nostra llista i en breu 
rebràs notícies nostres.""", "ca")
            if "es" in site_languages:
                confirmation_success_page.set("title", u"Página de confirmación", "es")
                confirmation_success_page.set("body", u"""Tu suscripción ha sido 
confirmada. Tu dirección ha sido añadida a nuestra lista y en breve recibirás 
noticias nuestras.""", "es")
            confirmation_success_page.parent = Site.main.home
            confirmation_success_page.template = standard_template
            confirmation_success_page.hidden = True
            confirmation_success_page.insert()

        # Default unsubscribe page
        unsubscribe_page = StandardPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.unsubscribe_page"
        )

        if unsubscribe_page is None and not extension.initialized:
            unsubscribe_page = StandardPage(
                qname = u"woost.extensions.campaignmonitor.unsubscribe_page"
            )
            if "en" in site_languages:
                unsubscribe_page.set("title", u"Unsubscribe page", "en")
                unsubscribe_page.set("body", u"""You have been successfully removed 
from this subscriber list. You will no longer hear from us.""", "en")
            if "ca" in site_languages:
                unsubscribe_page.set("title", u"Pàgina de baixa", "ca")
                unsubscribe_page.set("body", u"""La teva adreça ha sigut eliminada 
amb èxit d'aquesta llista de subscripció. No tornaràs a rebre comunicacions 
nostres.""", "ca")
            if "es" in site_languages:
                unsubscribe_page.set("title", u"Página de baja", "es")
                unsubscribe_page.set("body", u"""Tu dirección ha sido eliminada con 
éxito de esta lista de suscripción. No volverás a recibir comunicaciones 
nuestras.""", "es")
            unsubscribe_page.controller = unsubscription_controller
            unsubscribe_page.template = unsubscription_view
            unsubscribe_page.parent = Site.main.home
            unsubscribe_page.hidden = True
            unsubscribe_page.insert()

        extension.initialized = True
        datastore.commit()


    def synchronize_lists(self, restricted = False):                                                                                                                                                      
        """Synchronizes the list of Lists of the given Campaign Monitor account 
        with the site's database.

        The method queries Campaing Monitor API, retrieving the list of
        Lists for the indicated account and comparing it with the set of
        already known Lists (from previous executions of this method). The
        local database will be updated as follows:

            * Lists declared by Campaing Monitor that are not present in the 
              database will generate new instances.
            * Lists that exist on both ends will be updated with the data
              provided by the Campaing Monitor service (only non editable 
              members will be updated, so that data entered by users in the 
              backoffice is preserved).
            * Lists that were instantiated in a previous run but which have
              been deleted at the Campaign Monitor side will be removed from 
              the database.
                
        @param restricted: Indicated if access control should be applied to the
            operations performed by the method.
        @type restricted: bool
        """
        from cocktail.controllers import context
        from cocktail.controllers.location import Location
        from woost.extensions.campaignmonitor.campaignmonitorlist import \
            CampaignMonitorList
        from woost.extensions.campaignmonitor.campaign_monitor_api import \
            CampaignMonitorApi

        if restricted:
            user = get_current_user()

        api = CampaignMonitorApi(
            self.api_key,
            self.client_id
        )

        remote_lists = set()
        
        for list_data in api.client_get_lists():

            list_id = list_data["ListID"]
            cmlist = CampaignMonitorList.get_instance(list_id = list_id)
            remote_lists.add(list_id)

            # Check permissions
            if restricted and not user.has_permission(
                CreatePermission if cmlist is None else ModifyPermission,
                target = (cmlist or CampaignMonitorList)
            ):
                continue

            # Create new lists
            if cmlist is None:
                cmlist = CampaignMonitorList()
                cmlist.insert()
                cmlist.list_id = list_id
                self.lists.append(cmlist)

            # Modify new or updated lists with remote data
            cmlist.title = list_data.get("Name")

            list_detail_data = api.list_get_detail(list_id)

            # Modify remote lists with page urls

            def absolute_uri(publishable, *args, **kwargs):
                location = Location.get_current_host()
                location.path_info = context["cms"].uri(
                    publishable,
                    *args,
                    **kwargs
                )
                return str(location)

            unsubscribe_page = cmlist.unsubscribe_page and \
                "%s?user=[email]" % absolute_uri(cmlist.unsubscribe_page) or ""
            confirmation_success_page = cmlist.confirmation_success_page and \
                absolute_uri(cmlist.confirmation_success_page) or ""

            api.list_update(
                list_id,
                list_detail_data.get("Title"), 
                unsubscribe_page,
                list_detail_data.get("ConfirmOptIn"), 
                confirmation_success_page
            )

        # Delete lists that have been deleted from the user account
        missing_lists = CampaignMonitorList.select(
            CampaignMonitorList.list_id.not_one_of(remote_lists),
        )

        if restricted:
            missing_lists.add_filter(
                PermissionExpression(user, DeletePermission)
            )

        missing_lists.delete_items()



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cherrypy.lib.static import serve_file
from cocktail.controllers import RequestHandler, context


class FileHandler(RequestHandler):

    def __call__(self):
        document = context["document"]
        return serve_file(
                    document.file_path,
                    content_type = document.mime_type)

